use crate::algorithms::algorithm_trait::{Algorithm, Basic};
use crate::structures::binary_tree::{NodeData, Tree};
use crate::structures::structure_trait::Structure;
use crate::structures::structures_types::Support;

pub struct MurTree {}

impl Basic for MurTree {}

impl Algorithm for MurTree {
    fn build_depth_one_tree<S>(structure: &mut S, min_sup: Support) -> Tree<NodeData>
    where
        S: Structure,
    {
        let candidates = Self::generate_candidates_list(structure, min_sup);

        if candidates.is_empty() {
            return Self::empty_tree(1);
        }

        let mut tree = Self::empty_tree(1);
        let mut left_index = 0;
        let mut right_index = 0;
        if let Some(root) = tree.get_node_mut(tree.get_root_index()) {
            left_index = root.left;
            right_index = root.right
        }
        for candidate in candidates.iter() {
            structure.push((*candidate, 0));
            let classes_support = structure.labels_support();
            let left_error = Self::get_leaf_error(classes_support);
            structure.backtrack();

            structure.push((*candidate, 1));
            let classes_support = structure.labels_support();
            let right_error = Self::get_leaf_error(classes_support);
            structure.backtrack();

            let error = left_error.0 + right_error.0;
            let mut past_error = <usize>::MAX;

            if let Some(root) = tree.get_node_mut(tree.get_root_index()) {
                past_error = root.value.error;
            }
            if error < past_error {
                if let Some(root) = tree.get_node_mut(tree.get_root_index()) {
                    root.value.error = error;
                    root.value.test = Some(*candidate);
                }
                if let Some(left) = tree.get_node_mut(left_index) {
                    left.value.error = left_error.0;
                    left.value.out = Some(left_error.1);
                }
                if let Some(right) = tree.get_node_mut(right_index) {
                    right.value.error = right_error.0;
                    right.value.out = Some(right_error.1);
                }
            }
        }
        tree
    }

    fn build_depth_two_tree<S>(structure: &mut S, min_sup: Support) -> Tree<NodeData>
    where
        S: Structure,
    {
        // TODO : depth attribute
        let candidates = Self::generate_candidates_list(structure, min_sup);

        if candidates.is_empty() {
            return Self::empty_tree(2);
        }
        if candidates.len() < 2 {
            return Self::build_depth_one_tree(structure, min_sup);
        }

        let matrix = Self::build_depth_two_matrix(structure, &candidates);

        let mut tree = Self::empty_tree(2);
        let classes_support = structure.labels_support().to_vec();
        let support = structure.get_support();

        let before_creation_error = Self::get_leaf_error(&classes_support);

        for (i, first) in candidates.iter().enumerate() {
            // LEFT PART
            // println!("First : {}", first);
            // tree.print();

            let i_right_classes_support = &matrix[i][i];
            let i_left_classes_support =
                Self::get_diff_errors(&classes_support, i_right_classes_support);

            let root_right_support = matrix[i][i].iter().sum::<usize>();
            let root_left_support = support - root_right_support;
            // println!("Left sup : {}, right_sup {}", root_right_support, root_left_support);
            if root_left_support < min_sup || root_right_support < min_sup {
                continue;
            }

            // TODO : Should check if there is enough support for the leaf as root_left_support should be >= 2 * min_sup otherwise it is a leaf.

            let mut root_tree = Self::empty_tree(2);
            let mut left_index = 0;
            let mut right_index = 0;

            if let Some(root_node) = root_tree.get_node_mut(root_tree.get_root_index()) {
                root_node.value.test = Some(*first);
                root_node.value.error = before_creation_error.0;
                left_index = root_node.left;
                right_index = root_node.right;
            }

            // TODO : Check if support in enough 2 * min_sup

            if root_left_support < 2 * min_sup {
                let error = Self::get_leaf_error(&i_left_classes_support);
                if let Some(left_node) = root_tree.get_node_mut(left_index) {
                    left_node.value.error = error.0;
                    left_node.value.out = Some(error.1);
                    left_node.left = 0;
                    left_node.right = 0;
                }
                if let Some(node) = tree.get_node(tree.get_root_index()) {
                    if node.value.error < error.0 {
                        continue;
                    }
                }
            } else {
                let mut feat_error = <usize>::MAX;
                if let Some(root) = tree.get_node_mut(tree.get_root_index()) {
                    feat_error = root.value.error;
                }
                let error = Self::get_leaf_error(&i_left_classes_support);

                if error.0 < feat_error {
                    if let Some(left_node) = root_tree.get_node_mut(left_index) {
                        left_node.value.error = error.0;
                    }
                }

                for (j, second) in candidates.iter().enumerate().take(candidates.len()) {
                    if i == j {
                        // TODO: Can be replaced by first == second
                        continue;
                    }

                    // TODO: Add upper bound for left or right error based on the best tree so far

                    // Here is if the left part of the tree so the classes support of i__left -> j__right
                    let i_left_j_right_classes_support =
                        Self::get_diff_errors(&matrix[j][j], &matrix[i][j]);
                    let j_right_support = matrix[j][j].iter().sum::<usize>();
                    let i_right_j_right_support = matrix[i][j].iter().sum::<usize>();
                    let i_left_j_right_support = j_right_support - i_right_j_right_support; // Important
                    let i_left_j_left_support = root_left_support - i_left_j_right_support; // Important

                    if i_left_j_right_support < min_sup || i_left_j_left_support < min_sup {
                        // Not enough support for the left part of the tree
                        continue;
                    }

                    let right_leaf_error = Self::get_leaf_error(&i_left_j_right_classes_support);

                    // TODO Upper bound control here
                    if right_leaf_error.0 >= feat_error {
                        continue;
                    }

                    let i_left_j_left_classes_support = Self::get_diff_errors(
                        &i_left_classes_support,
                        &i_left_j_right_classes_support,
                    );
                    let left_leaf_error = Self::get_leaf_error(&i_left_j_left_classes_support);

                    // TODO Upper bound control here
                    // println!("Feature error : {:?}", feat_error);
                    // println!("First : {}", first);
                    // if *first == 5 {
                    //     println!("First attribute : {:?} Second attribute : {:?} Left error : {:?} Right error : {:?}\n", first, second, left_leaf_error.0, right_leaf_error.0);
                    // }

                    if (left_leaf_error.0 + right_leaf_error.0) >= feat_error {
                        continue;
                    }

                    let mut left_leaf_index = 0;
                    let mut right_leaf_index = 0;

                    if let Some(left_node) = root_tree.get_node_mut(left_index) {
                        left_node.value.test = Some(*second);
                        left_node.value.error = left_leaf_error.0 + right_leaf_error.0;
                        left_leaf_index = left_node.left;
                        right_leaf_index = left_node.right;
                    }

                    if let Some(left_leaf) = root_tree.get_node_mut(left_leaf_index) {
                        left_leaf.value.error = left_leaf_error.0;
                        left_leaf.value.out = Some(left_leaf_error.1);
                    }

                    if let Some(right_leaf) = root_tree.get_node_mut(right_leaf_index) {
                        right_leaf.value.error = right_leaf_error.0;
                        right_leaf.value.out = Some(right_leaf_error.1);
                    }

                    feat_error = left_leaf_error.0 + right_leaf_error.0;

                    if feat_error == 0 {
                        break;
                    }
                }
            }

            // println!("Root tree Before going to the right");
            // root_tree.print();

            if root_right_support < 2 * min_sup {
                let error = Self::get_leaf_error(i_right_classes_support);
                if let Some(right_node) = root_tree.get_node_mut(right_index) {
                    right_node.value.error = error.0;
                    right_node.value.out = Some(error.1);
                    right_node.left = 0;
                    right_node.right = 0;
                }

                let mut current_left_error = <usize>::MAX;
                let best_error = tree.get_node(tree.get_root_index()).unwrap().value.error;

                if let Some(node) = root_tree.get_node(left_index) {
                    current_left_error = node.value.error;
                }
                if current_left_error > best_error || error.0 >= best_error - current_left_error {
                    // TODO Danger here
                    continue;
                }
            } else {
                let mut feat_error = <usize>::MAX;
                if let Some(root) = tree.get_node(tree.get_root_index()) {
                    feat_error = root.value.error;
                }
                // println!("Before feat_error: {}", feat_error);
                let mut current_left_error = <usize>::MAX;
                //root_tree.print();
                if let Some(left_node) = root_tree.get_node(left_index) {
                    current_left_error = left_node.value.error;
                    // feat_error = feat_error.saturating_sub(left_node.value.error);
                }

                // println!("After feat_error: {}", feat_error);

                let error = Self::get_leaf_error(i_right_classes_support);
                // println!("Error: {:?}", error);
                // println!("I right classes support: {:?}", i_right_classes_support);
                if current_left_error > feat_error || error.0 < feat_error - current_left_error {
                    if let Some(right_node) = root_tree.get_node_mut(right_index) {
                        right_node.value.error = error.0;
                        right_node.value.out = Some(error.1);
                    }
                }

                // TODO LOWER BOUND CONTROL HERE
                // if

                for (j, second) in candidates.iter().enumerate().take(candidates.len()) {
                    if i == j {
                        // TODO: Can be replaced by first == second
                        continue;
                    }

                    // Here is if the right part of the tree so the classes support of i__right -> j__right
                    let i_right_j_right_classes_support = &matrix[i][j];
                    let i_right_j_left_classes_support =
                        Self::get_diff_errors(&matrix[i][i], &matrix[i][j]);
                    let i_right_j_right_support = matrix[i][j].iter().sum::<usize>();
                    let i_right_j_left_support = root_right_support - i_right_j_right_support; // Important

                    if i_right_j_left_support < min_sup || i_right_j_right_support < min_sup {
                        // Not enough support for the right part of the tree
                        continue;
                    }

                    let left_leaf_error = Self::get_leaf_error(&i_right_j_left_classes_support);

                    // TODO Upper bound control here
                    if left_leaf_error.0 >= feat_error {
                        continue;
                    }

                    let right_leaf_error = Self::get_leaf_error(i_right_j_right_classes_support);

                    // TODO Upper bound control here
                    if (left_leaf_error.0 + right_leaf_error.0) >= feat_error {
                        continue;
                    }

                    let mut left_leaf_index = 0;
                    let mut right_leaf_index = 0;

                    if let Some(right_node) = root_tree.get_node_mut(right_index) {
                        right_node.value.test = Some(*second);
                        right_node.value.error = left_leaf_error.0 + right_leaf_error.0;
                        left_leaf_index = right_node.left;
                        right_leaf_index = right_node.right;
                    }

                    if let Some(left_leaf) = root_tree.get_node_mut(left_leaf_index) {
                        left_leaf.value.error = left_leaf_error.0;
                        left_leaf.value.out = Some(left_leaf_error.1);
                    }

                    if let Some(right_leaf) = root_tree.get_node_mut(right_leaf_index) {
                        right_leaf.value.error = right_leaf_error.0;
                        right_leaf.value.out = Some(right_leaf_error.1);
                    }

                    feat_error = left_leaf_error.0 + right_leaf_error.0;
                    if feat_error == 0 {
                        break;
                    }
                }

                let mut feat_error = <usize>::MAX;
                if let Some(node) = root_tree.get_node(left_index) {
                    feat_error = node.value.error;
                }
                if let Some(node) = root_tree.get_node(right_index) {
                    feat_error = feat_error.saturating_add(node.value.error);
                }

                if let Some(feat_tree) = root_tree.get_node_mut(tree.get_root_index()) {
                    feat_tree.value.error = feat_error;
                }

                let mut best_error = <usize>::MAX;
                if let Some(node) = tree.get_node(tree.get_root_index()) {
                    best_error = node.value.error;
                }
                // println!("Best error: {}", best_error);
                // println!("Feat error: {}", feat_error);
                // println!("------------------");
                if best_error > feat_error {
                    tree = root_tree;
                }
                // tree.print();
                if feat_error == 0 {
                    break;
                }
            }
            // tree.print();
        }

        tree
    }
}

#[cfg(test)]
mod murtree_test {
    use crate::algorithms::algorithm_trait::Algorithm;
    use crate::algorithms::murtree::MurTree;
    use crate::dataset::binary_dataset::BinaryDataset;
    use crate::dataset::data_trait::Dataset;
    use crate::structures::bitsets_structure::BitsetStructure;
    use crate::structures::horizontal_binary_structure::HorizontalBinaryStructure;
    use crate::structures::reversible_sparse_bitsets_structure::RSparseBitsetStructure;
    use crate::structures::structure_trait::Structure;
    use crate::structures::structures_types::{BitsetStructData, HorizontalData};

    // #[test]
    // fn test_full_data_matrix_building() {
    //     // TODO: Move this test to trait test
    //
    //     let dataset = BinaryDataset::load("test_data/anneal.txt", false, 0.0);
    //     let bitset_data = RSparseBitsetStructure::format_input_data(&dataset);
    //     let mut structure = RSparseBitsetStructure::new(&bitset_data);
    //     let candidates = MurTree::generate_candidates_list(&mut structure, 1);
    //
    //     let num_attributes = structure.num_attributes();
    //
    //     let expected_matrix = [
    //         [
    //             (72usize, 184usize),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (72, 173),
    //             (0, 11),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (70, 179),
    //             (2, 5),
    //             (72, 179),
    //             (0, 5),
    //             (72, 179),
    //             (0, 5),
    //             (72, 179),
    //             (0, 5),
    //             (72, 179),
    //             (0, 5),
    //             (72, 184),
    //             (0, 0),
    //             (72, 184),
    //             (0, 0),
    //             (12, 70),
    //             (60, 114),
    //             (0, 22),
    //             (72, 162),
    //             (0, 39),
    //             (72, 145),
    //             (0, 69),
    //             (72, 115),
    //             (0, 95),
    //             (72, 89),
    //             (30, 112),
    //             (42, 72),
    //             (50, 132),
    //             (22, 52),
    //             (72, 153),
    //             (0, 31),
    //             (5, 17),
    //             (67, 167),
    //             (10, 55),
    //             (62, 129),
    //             (37, 115),
    //             (35, 69),
    //             (44, 121),
    //             (28, 63),
    //             (44, 130),
    //             (28, 54),
    //             (49, 142),
    //             (23, 42),
    //             (60, 184),
    //             (12, 0),
    //             (12, 70),
    //             (60, 114),
    //             (23, 86),
    //             (49, 98),
    //             (25, 90),
    //             (47, 94),
    //             (41, 135),
    //             (31, 49),
    //             (58, 146),
    //             (14, 38),
    //             (72, 179),
    //             (0, 0),
    //             (0, 5),
    //         ],
    //         [
    //             (0, 0),
    //             (89, 351),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (86, 344),
    //             (3, 7),
    //             (87, 348),
    //             (2, 3),
    //             (89, 351),
    //             (0, 0),
    //             (89, 351),
    //             (0, 0),
    //             (89, 351),
    //             (0, 0),
    //             (89, 351),
    //             (0, 0),
    //             (51, 228),
    //             (38, 123),
    //             (51, 284),
    //             (38, 67),
    //             (51, 310),
    //             (38, 41),
    //             (51, 336),
    //             (38, 15),
    //             (51, 349),
    //             (38, 2),
    //             (54, 351),
    //             (35, 0),
    //             (89, 346),
    //             (0, 5),
    //             (89, 346),
    //             (0, 5),
    //             (89, 346),
    //             (0, 5),
    //             (89, 346),
    //             (0, 5),
    //             (89, 346),
    //             (0, 5),
    //             (89, 351),
    //             (0, 0),
    //             (89, 351),
    //             (0, 0),
    //             (43, 165),
    //             (46, 186),
    //             (12, 57),
    //             (77, 294),
    //             (31, 114),
    //             (58, 237),
    //             (31, 162),
    //             (58, 189),
    //             (32, 225),
    //             (57, 126),
    //             (39, 265),
    //             (50, 86),
    //             (54, 289),
    //             (35, 62),
    //             (75, 318),
    //             (14, 33),
    //             (19, 53),
    //             (70, 298),
    //             (37, 112),
    //             (52, 239),
    //             (62, 228),
    //             (27, 123),
    //             (65, 230),
    //             (24, 121),
    //             (86, 265),
    //             (3, 86),
    //             (86, 274),
    //             (3, 77),
    //             (89, 346),
    //             (0, 5),
    //             (44, 169),
    //             (45, 182),
    //             (59, 187),
    //             (30, 164),
    //             (64, 199),
    //             (25, 152),
    //             (74, 267),
    //             (15, 84),
    //             (79, 278),
    //             (10, 73),
    //             (83, 340),
    //             (2, 1),
    //             (4, 10),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 17),
    //             (0, 34),
    //             (0, 36),
    //             (0, 15),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 0),
    //             (0, 51),
    //             (0, 5),
    //             (0, 46),
    //             (0, 5),
    //             (0, 46),
    //             (0, 5),
    //             (0, 46),
    //             (0, 12),
    //             (0, 39),
    //             (0, 27),
    //             (0, 24),
    //             (0, 32),
    //             (0, 19),
    //             (0, 1),
    //             (0, 50),
    //             (0, 1),
    //             (0, 50),
    //             (0, 11),
    //             (0, 40),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 35),
    //             (0, 6),
    //             (0, 10),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 9),
    //             (11, 0),
    //             (0, 9),
    //             (11, 0),
    //             (0, 9),
    //             (11, 0),
    //             (11, 9),
    //             (0, 0),
    //             (4, 9),
    //             (7, 0),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (0, 0),
    //             (11, 9),
    //             (4, 2),
    //             (7, 7),
    //             (8, 5),
    //             (3, 4),
    //             (11, 9),
    //             (0, 0),
    //             (0, 1),
    //             (11, 8),
    //             (0, 4),
    //             (11, 5),
    //             (2, 6),
    //             (9, 3),
    //             (10, 6),
    //             (1, 3),
    //             (10, 6),
    //             (1, 3),
    //             (10, 7),
    //             (1, 2),
    //             (11, 9),
    //             (0, 0),
    //             (4, 9),
    //             (7, 0),
    //             (4, 9),
    //             (7, 0),
    //             (4, 9),
    //             (7, 0),
    //             (4, 9),
    //             (7, 0),
    //             (10, 9),
    //             (1, 0),
    //             (11, 9),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (0, 0),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 5),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (4, 5),
    //             (1, 0),
    //             (4, 5),
    //             (1, 0),
    //             (4, 5),
    //             (1, 0),
    //             (5, 1),
    //             (0, 4),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (5, 5),
    //             (0, 0),
    //             (5, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 0),
    //             (0, 5),
    //             (5, 2),
    //             (0, 3),
    //             (3, 0),
    //             (2, 5),
    //             (5, 1),
    //             (0, 4),
    //             (5, 3),
    //             (0, 2),
    //             (5, 4),
    //             (0, 1),
    //             (5, 4),
    //             (0, 1),
    //             (5, 4),
    //             (0, 1),
    //             (5, 5),
    //             (0, 0),
    //             (5, 1),
    //             (0, 4),
    //             (5, 1),
    //             (0, 4),
    //             (5, 1),
    //             (0, 4),
    //             (5, 3),
    //             (0, 2),
    //             (5, 3),
    //             (0, 2),
    //             (5, 5),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 7),
    //             (0, 12),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 4),
    //             (0, 15),
    //             (0, 7),
    //             (0, 12),
    //             (0, 10),
    //             (0, 9),
    //             (0, 0),
    //             (0, 19),
    //             (0, 2),
    //             (0, 17),
    //             (0, 9),
    //             (0, 10),
    //             (0, 9),
    //             (0, 10),
    //             (0, 9),
    //             (0, 10),
    //             (0, 19),
    //             (0, 0),
    //             (0, 19),
    //             (0, 0),
    //             (0, 7),
    //             (0, 12),
    //             (0, 7),
    //             (0, 12),
    //             (0, 7),
    //             (0, 12),
    //             (0, 15),
    //             (0, 4),
    //             (0, 15),
    //             (0, 4),
    //             (0, 15),
    //             (0, 2),
    //             (0, 2),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (10, 6),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (3, 3),
    //             (7, 3),
    //             (0, 0),
    //             (10, 6),
    //             (0, 0),
    //             (10, 6),
    //             (0, 1),
    //             (10, 5),
    //             (0, 4),
    //             (10, 2),
    //             (5, 5),
    //             (5, 1),
    //             (9, 5),
    //             (1, 1),
    //             (10, 5),
    //             (0, 1),
    //             (2, 0),
    //             (8, 6),
    //             (3, 2),
    //             (7, 4),
    //             (5, 3),
    //             (5, 3),
    //             (5, 3),
    //             (5, 3),
    //             (5, 3),
    //             (5, 3),
    //             (5, 3),
    //             (5, 3),
    //             (8, 6),
    //             (2, 0),
    //             (3, 3),
    //             (7, 3),
    //             (5, 3),
    //             (5, 3),
    //             (5, 3),
    //             (5, 3),
    //             (8, 5),
    //             (2, 1),
    //             (8, 5),
    //             (2, 1),
    //             (10, 6),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (72, 173),
    //             (86, 344),
    //             (0, 0),
    //             (11, 9),
    //             (5, 0),
    //             (0, 19),
    //             (10, 6),
    //             (184, 551),
    //             (0, 0),
    //             (184, 551),
    //             (0, 0),
    //             (184, 551),
    //             (0, 0),
    //             (184, 551),
    //             (0, 0),
    //             (184, 551),
    //             (0, 0),
    //             (184, 551),
    //             (0, 0),
    //             (146, 428),
    //             (38, 123),
    //             (146, 484),
    //             (38, 67),
    //             (146, 510),
    //             (38, 41),
    //             (146, 536),
    //             (38, 15),
    //             (146, 549),
    //             (38, 2),
    //             (149, 551),
    //             (35, 0),
    //             (166, 513),
    //             (18, 38),
    //             (168, 513),
    //             (16, 38),
    //             (168, 532),
    //             (16, 19),
    //             (168, 541),
    //             (16, 10),
    //             (172, 541),
    //             (12, 10),
    //             (172, 551),
    //             (12, 0),
    //             (183, 551),
    //             (1, 0),
    //             (64, 243),
    //             (120, 308),
    //             (11, 75),
    //             (173, 476),
    //             (30, 147),
    //             (154, 404),
    //             (30, 225),
    //             (154, 326),
    //             (31, 316),
    //             (153, 235),
    //             (82, 372),
    //             (102, 179),
    //             (124, 421),
    //             (60, 130),
    //             (171, 478),
    //             (13, 73),
    //             (29, 67),
    //             (155, 484),
    //             (54, 170),
    //             (130, 381),
    //             (109, 349),
    //             (75, 202),
    //             (127, 357),
    //             (57, 194),
    //             (148, 398),
    //             (36, 153),
    //             (153, 430),
    //             (31, 121),
    //             (170, 546),
    //             (14, 5),
    //             (65, 247),
    //             (119, 304),
    //             (93, 281),
    //             (91, 270),
    //             (100, 297),
    //             (84, 254),
    //             (129, 416),
    //             (55, 135),
    //             (157, 438),
    //             (27, 113),
    //             (178, 531),
    //             (2, 3),
    //             (4, 17),
    //         ],
    //         [
    //             (0, 11),
    //             (3, 7),
    //             (0, 51),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (3, 74),
    //             (1, 15),
    //             (2, 59),
    //             (3, 18),
    //             (0, 56),
    //             (3, 35),
    //             (0, 39),
    //             (3, 54),
    //             (0, 20),
    //             (3, 69),
    //             (0, 5),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 74),
    //             (0, 0),
    //             (3, 63),
    //             (0, 11),
    //             (1, 4),
    //             (2, 70),
    //             (1, 11),
    //             (2, 63),
    //             (1, 12),
    //             (2, 62),
    //             (1, 13),
    //             (2, 61),
    //             (1, 28),
    //             (2, 46),
    //             (2, 44),
    //             (1, 30),
    //             (2, 51),
    //             (1, 23),
    //             (0, 5),
    //             (3, 69),
    //             (1, 7),
    //             (2, 67),
    //             (2, 26),
    //             (1, 48),
    //             (2, 67),
    //             (1, 7),
    //             (2, 70),
    //             (1, 4),
    //             (2, 70),
    //             (1, 4),
    //             (3, 74),
    //             (0, 0),
    //             (3, 63),
    //             (0, 11),
    //             (3, 63),
    //             (0, 11),
    //             (3, 63),
    //             (0, 11),
    //             (3, 69),
    //             (0, 5),
    //             (3, 69),
    //             (0, 5),
    //             (3, 58),
    //             (0, 6),
    //             (0, 10),
    //         ],
    //         [
    //             (72, 184),
    //             (87, 348),
    //             (0, 0),
    //             (11, 9),
    //             (5, 0),
    //             (0, 19),
    //             (10, 6),
    //             (184, 551),
    //             (1, 15),
    //             (185, 566),
    //             (0, 0),
    //             (185, 566),
    //             (0, 0),
    //             (185, 566),
    //             (0, 0),
    //             (185, 566),
    //             (0, 0),
    //             (185, 566),
    //             (0, 0),
    //             (147, 443),
    //             (38, 123),
    //             (147, 499),
    //             (38, 67),
    //             (147, 525),
    //             (38, 41),
    //             (147, 551),
    //             (38, 15),
    //             (147, 564),
    //             (38, 2),
    //             (150, 566),
    //             (35, 0),
    //             (167, 528),
    //             (18, 38),
    //             (169, 528),
    //             (16, 38),
    //             (169, 547),
    //             (16, 19),
    //             (169, 556),
    //             (16, 10),
    //             (173, 556),
    //             (12, 10),
    //             (173, 566),
    //             (12, 0),
    //             (184, 566),
    //             (1, 0),
    //             (65, 251),
    //             (120, 315),
    //             (12, 79),
    //             (173, 487),
    //             (31, 153),
    //             (154, 413),
    //             (31, 231),
    //             (154, 335),
    //             (32, 322),
    //             (153, 244),
    //             (83, 386),
    //             (102, 180),
    //             (125, 436),
    //             (60, 130),
    //             (172, 493),
    //             (13, 73),
    //             (29, 70),
    //             (156, 496),
    //             (54, 173),
    //             (131, 393),
    //             (110, 358),
    //             (75, 208),
    //             (128, 366),
    //             (57, 200),
    //             (149, 410),
    //             (36, 156),
    //             (154, 442),
    //             (31, 124),
    //             (171, 561),
    //             (14, 5),
    //             (66, 255),
    //             (119, 311),
    //             (94, 289),
    //             (91, 277),
    //             (101, 305),
    //             (84, 261),
    //             (130, 428),
    //             (55, 138),
    //             (158, 450),
    //             (27, 116),
    //             (179, 546),
    //             (2, 3),
    //             (4, 17),
    //         ],
    //         [
    //             (0, 0),
    //             (2, 3),
    //             (0, 51),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (2, 3),
    //             (0, 56),
    //             (2, 20),
    //             (0, 39),
    //             (2, 39),
    //             (0, 20),
    //             (2, 54),
    //             (0, 5),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 59),
    //             (0, 0),
    //             (2, 55),
    //             (0, 4),
    //             (0, 0),
    //             (2, 59),
    //             (0, 5),
    //             (2, 54),
    //             (0, 6),
    //             (2, 53),
    //             (0, 7),
    //             (2, 52),
    //             (0, 14),
    //             (2, 45),
    //             (1, 29),
    //             (1, 30),
    //             (1, 36),
    //             (1, 23),
    //             (0, 2),
    //             (2, 57),
    //             (1, 4),
    //             (1, 55),
    //             (1, 17),
    //             (1, 42),
    //             (1, 58),
    //             (1, 1),
    //             (1, 58),
    //             (1, 1),
    //             (1, 58),
    //             (1, 1),
    //             (2, 59),
    //             (0, 0),
    //             (2, 55),
    //             (0, 4),
    //             (2, 55),
    //             (0, 4),
    //             (2, 55),
    //             (0, 4),
    //             (2, 57),
    //             (0, 2),
    //             (2, 57),
    //             (0, 2),
    //             (2, 43),
    //             (0, 6),
    //             (0, 10),
    //         ],
    //         [
    //             (72, 184),
    //             (89, 351),
    //             (0, 0),
    //             (11, 9),
    //             (5, 0),
    //             (0, 19),
    //             (10, 6),
    //             (184, 551),
    //             (3, 18),
    //             (185, 566),
    //             (2, 3),
    //             (187, 569),
    //             (0, 0),
    //             (187, 569),
    //             (0, 0),
    //             (187, 569),
    //             (0, 0),
    //             (187, 569),
    //             (0, 0),
    //             (149, 446),
    //             (38, 123),
    //             (149, 502),
    //             (38, 67),
    //             (149, 528),
    //             (38, 41),
    //             (149, 554),
    //             (38, 15),
    //             (149, 567),
    //             (38, 2),
    //             (152, 569),
    //             (35, 0),
    //             (169, 531),
    //             (18, 38),
    //             (171, 531),
    //             (16, 38),
    //             (171, 550),
    //             (16, 19),
    //             (171, 559),
    //             (16, 10),
    //             (175, 559),
    //             (12, 10),
    //             (175, 569),
    //             (12, 0),
    //             (186, 569),
    //             (1, 0),
    //             (67, 254),
    //             (120, 315),
    //             (12, 79),
    //             (175, 490),
    //             (31, 153),
    //             (156, 416),
    //             (31, 232),
    //             (156, 337),
    //             (32, 324),
    //             (155, 245),
    //             (83, 388),
    //             (104, 181),
    //             (126, 438),
    //             (61, 131),
    //             (173, 495),
    //             (14, 74),
    //             (29, 71),
    //             (158, 498),
    //             (55, 175),
    //             (132, 394),
    //             (111, 361),
    //             (76, 208),
    //             (129, 369),
    //             (58, 200),
    //             (150, 413),
    //             (37, 156),
    //             (155, 445),
    //             (32, 124),
    //             (173, 564),
    //             (14, 5),
    //             (68, 258),
    //             (119, 311),
    //             (96, 292),
    //             (91, 277),
    //             (103, 308),
    //             (84, 261),
    //             (132, 431),
    //             (55, 138),
    //             (160, 453),
    //             (27, 116),
    //             (181, 549),
    //             (2, 3),
    //             (4, 17),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 51),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 17),
    //             (0, 39),
    //             (0, 36),
    //             (0, 20),
    //             (0, 51),
    //             (0, 5),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 56),
    //             (0, 0),
    //             (0, 52),
    //             (0, 4),
    //             (0, 0),
    //             (0, 56),
    //             (0, 5),
    //             (0, 51),
    //             (0, 5),
    //             (0, 51),
    //             (0, 5),
    //             (0, 51),
    //             (0, 12),
    //             (0, 44),
    //             (0, 27),
    //             (0, 29),
    //             (0, 34),
    //             (0, 22),
    //             (0, 1),
    //             (0, 55),
    //             (0, 2),
    //             (0, 54),
    //             (0, 14),
    //             (0, 42),
    //             (0, 55),
    //             (0, 1),
    //             (0, 55),
    //             (0, 1),
    //             (0, 55),
    //             (0, 1),
    //             (0, 56),
    //             (0, 0),
    //             (0, 52),
    //             (0, 4),
    //             (0, 52),
    //             (0, 4),
    //             (0, 52),
    //             (0, 4),
    //             (0, 54),
    //             (0, 2),
    //             (0, 54),
    //             (0, 2),
    //             (0, 40),
    //             (0, 6),
    //             (0, 10),
    //         ],
    //         [
    //             (72, 184),
    //             (89, 351),
    //             (0, 17),
    //             (11, 9),
    //             (5, 0),
    //             (0, 19),
    //             (10, 6),
    //             (184, 551),
    //             (3, 35),
    //             (185, 566),
    //             (2, 20),
    //             (187, 569),
    //             (0, 17),
    //             (187, 586),
    //             (0, 0),
    //             (187, 586),
    //             (0, 0),
    //             (187, 586),
    //             (0, 0),
    //             (149, 463),
    //             (38, 123),
    //             (149, 519),
    //             (38, 67),
    //             (149, 545),
    //             (38, 41),
    //             (149, 571),
    //             (38, 15),
    //             (149, 584),
    //             (38, 2),
    //             (152, 586),
    //             (35, 0),
    //             (169, 548),
    //             (18, 38),
    //             (171, 548),
    //             (16, 38),
    //             (171, 567),
    //             (16, 19),
    //             (171, 576),
    //             (16, 10),
    //             (175, 576),
    //             (12, 10),
    //             (175, 586),
    //             (12, 0),
    //             (186, 586),
    //             (1, 0),
    //             (67, 271),
    //             (120, 315),
    //             (12, 79),
    //             (175, 507),
    //             (31, 157),
    //             (156, 429),
    //             (31, 236),
    //             (156, 350),
    //             (32, 328),
    //             (155, 258),
    //             (83, 392),
    //             (104, 194),
    //             (126, 447),
    //             (61, 139),
    //             (173, 506),
    //             (14, 80),
    //             (29, 71),
    //             (158, 515),
    //             (55, 175),
    //             (132, 411),
    //             (111, 365),
    //             (76, 221),
    //             (129, 386),
    //             (58, 200),
    //             (150, 430),
    //             (37, 156),
    //             (155, 462),
    //             (32, 124),
    //             (173, 581),
    //             (14, 5),
    //             (68, 275),
    //             (119, 311),
    //             (96, 309),
    //             (91, 277),
    //             (103, 325),
    //             (84, 261),
    //             (132, 448),
    //             (55, 138),
    //             (160, 470),
    //             (27, 116),
    //             (181, 562),
    //             (2, 4),
    //             (4, 20),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 34),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 19),
    //             (0, 20),
    //             (0, 34),
    //             (0, 5),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 39),
    //             (0, 0),
    //             (0, 35),
    //             (0, 4),
    //             (0, 0),
    //             (0, 39),
    //             (0, 1),
    //             (0, 38),
    //             (0, 1),
    //             (0, 38),
    //             (0, 1),
    //             (0, 38),
    //             (0, 8),
    //             (0, 31),
    //             (0, 18),
    //             (0, 21),
    //             (0, 23),
    //             (0, 16),
    //             (0, 1),
    //             (0, 38),
    //             (0, 2),
    //             (0, 37),
    //             (0, 10),
    //             (0, 29),
    //             (0, 38),
    //             (0, 1),
    //             (0, 38),
    //             (0, 1),
    //             (0, 38),
    //             (0, 1),
    //             (0, 39),
    //             (0, 0),
    //             (0, 35),
    //             (0, 4),
    //             (0, 35),
    //             (0, 4),
    //             (0, 35),
    //             (0, 4),
    //             (0, 37),
    //             (0, 2),
    //             (0, 37),
    //             (0, 2),
    //             (0, 27),
    //             (0, 5),
    //             (0, 7),
    //         ],
    //         [
    //             (72, 184),
    //             (89, 351),
    //             (0, 36),
    //             (11, 9),
    //             (5, 0),
    //             (0, 19),
    //             (10, 6),
    //             (184, 551),
    //             (3, 54),
    //             (185, 566),
    //             (2, 39),
    //             (187, 569),
    //             (0, 36),
    //             (187, 586),
    //             (0, 19),
    //             (187, 605),
    //             (0, 0),
    //             (187, 605),
    //             (0, 0),
    //             (149, 482),
    //             (38, 123),
    //             (149, 538),
    //             (38, 67),
    //             (149, 564),
    //             (38, 41),
    //             (149, 590),
    //             (38, 15),
    //             (149, 603),
    //             (38, 2),
    //             (152, 605),
    //             (35, 0),
    //             (169, 567),
    //             (18, 38),
    //             (171, 567),
    //             (16, 38),
    //             (171, 586),
    //             (16, 19),
    //             (171, 595),
    //             (16, 10),
    //             (175, 595),
    //             (12, 10),
    //             (175, 605),
    //             (12, 0),
    //             (186, 605),
    //             (1, 0),
    //             (67, 290),
    //             (120, 315),
    //             (12, 79),
    //             (175, 526),
    //             (31, 157),
    //             (156, 448),
    //             (31, 236),
    //             (156, 369),
    //             (32, 328),
    //             (155, 277),
    //             (83, 396),
    //             (104, 209),
    //             (126, 456),
    //             (61, 149),
    //             (173, 518),
    //             (14, 87),
    //             (29, 71),
    //             (158, 534),
    //             (55, 175),
    //             (132, 430),
    //             (111, 367),
    //             (76, 238),
    //             (129, 405),
    //             (58, 200),
    //             (150, 449),
    //             (37, 156),
    //             (155, 481),
    //             (32, 124),
    //             (173, 600),
    //             (14, 5),
    //             (68, 294),
    //             (119, 311),
    //             (96, 328),
    //             (91, 277),
    //             (103, 344),
    //             (84, 261),
    //             (132, 467),
    //             (55, 138),
    //             (160, 489),
    //             (27, 116),
    //             (181, 574),
    //             (2, 6),
    //             (4, 25),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 15),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 15),
    //             (0, 5),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 20),
    //             (0, 0),
    //             (0, 16),
    //             (0, 4),
    //             (0, 0),
    //             (0, 20),
    //             (0, 1),
    //             (0, 19),
    //             (0, 1),
    //             (0, 19),
    //             (0, 1),
    //             (0, 19),
    //             (0, 4),
    //             (0, 16),
    //             (0, 9),
    //             (0, 11),
    //             (0, 11),
    //             (0, 9),
    //             (0, 1),
    //             (0, 19),
    //             (0, 2),
    //             (0, 18),
    //             (0, 8),
    //             (0, 12),
    //             (0, 19),
    //             (0, 1),
    //             (0, 19),
    //             (0, 1),
    //             (0, 19),
    //             (0, 1),
    //             (0, 20),
    //             (0, 0),
    //             (0, 16),
    //             (0, 4),
    //             (0, 16),
    //             (0, 4),
    //             (0, 16),
    //             (0, 4),
    //             (0, 18),
    //             (0, 2),
    //             (0, 18),
    //             (0, 2),
    //             (0, 15),
    //             (0, 3),
    //             (0, 2),
    //         ],
    //         [
    //             (72, 184),
    //             (89, 351),
    //             (0, 51),
    //             (11, 9),
    //             (5, 0),
    //             (0, 19),
    //             (10, 6),
    //             (184, 551),
    //             (3, 69),
    //             (185, 566),
    //             (2, 54),
    //             (187, 569),
    //             (0, 51),
    //             (187, 586),
    //             (0, 34),
    //             (187, 605),
    //             (0, 15),
    //             (187, 620),
    //             (0, 0),
    //             (149, 497),
    //             (38, 123),
    //             (149, 553),
    //             (38, 67),
    //             (149, 579),
    //             (38, 41),
    //             (149, 605),
    //             (38, 15),
    //             (149, 618),
    //             (38, 2),
    //             (152, 620),
    //             (35, 0),
    //             (169, 582),
    //             (18, 38),
    //             (171, 582),
    //             (16, 38),
    //             (171, 601),
    //             (16, 19),
    //             (171, 610),
    //             (16, 10),
    //             (175, 610),
    //             (12, 10),
    //             (175, 620),
    //             (12, 0),
    //             (186, 620),
    //             (1, 0),
    //             (67, 305),
    //             (120, 315),
    //             (12, 79),
    //             (175, 541),
    //             (31, 158),
    //             (156, 462),
    //             (31, 237),
    //             (156, 383),
    //             (32, 329),
    //             (155, 291),
    //             (83, 400),
    //             (104, 220),
    //             (126, 465),
    //             (61, 155),
    //             (173, 527),
    //             (14, 93),
    //             (29, 72),
    //             (158, 548),
    //             (55, 176),
    //             (132, 444),
    //             (111, 372),
    //             (76, 248),
    //             (129, 420),
    //             (58, 200),
    //             (150, 464),
    //             (37, 156),
    //             (155, 496),
    //             (32, 124),
    //             (173, 615),
    //             (14, 5),
    //             (68, 309),
    //             (119, 311),
    //             (96, 343),
    //             (91, 277),
    //             (103, 359),
    //             (84, 261),
    //             (132, 482),
    //             (55, 138),
    //             (160, 504),
    //             (27, 116),
    //             (181, 584),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 1),
    //             (0, 4),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 5),
    //             (0, 2),
    //             (0, 3),
    //             (0, 0),
    //             (0, 5),
    //             (0, 1),
    //             (0, 4),
    //             (0, 3),
    //             (0, 2),
    //             (0, 4),
    //             (0, 1),
    //             (0, 4),
    //             (0, 1),
    //             (0, 4),
    //             (0, 1),
    //             (0, 5),
    //             (0, 0),
    //             (0, 1),
    //             (0, 4),
    //             (0, 1),
    //             (0, 4),
    //             (0, 1),
    //             (0, 4),
    //             (0, 3),
    //             (0, 2),
    //             (0, 3),
    //             (0, 2),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (72, 184),
    //             (51, 228),
    //             (0, 51),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 6),
    //             (146, 428),
    //             (3, 74),
    //             (147, 443),
    //             (2, 59),
    //             (149, 446),
    //             (0, 56),
    //             (149, 463),
    //             (0, 39),
    //             (149, 482),
    //             (0, 20),
    //             (149, 497),
    //             (0, 5),
    //             (149, 502),
    //             (0, 0),
    //             (149, 502),
    //             (0, 0),
    //             (149, 502),
    //             (0, 0),
    //             (149, 502),
    //             (0, 0),
    //             (149, 502),
    //             (0, 0),
    //             (149, 502),
    //             (0, 0),
    //             (131, 464),
    //             (18, 38),
    //             (133, 464),
    //             (16, 38),
    //             (133, 483),
    //             (16, 19),
    //             (133, 492),
    //             (16, 10),
    //             (137, 492),
    //             (12, 10),
    //             (137, 502),
    //             (12, 0),
    //             (148, 502),
    //             (1, 0),
    //             (46, 236),
    //             (103, 266),
    //             (3, 50),
    //             (146, 452),
    //             (22, 111),
    //             (127, 391),
    //             (22, 178),
    //             (127, 324),
    //             (23, 252),
    //             (126, 250),
    //             (69, 306),
    //             (80, 196),
    //             (103, 367),
    //             (46, 135),
    //             (148, 417),
    //             (1, 85),
    //             (24, 48),
    //             (125, 454),
    //             (43, 142),
    //             (106, 360),
    //             (89, 297),
    //             (60, 205),
    //             (104, 345),
    //             (45, 157),
    //             (114, 373),
    //             (35, 129),
    //             (119, 403),
    //             (30, 99),
    //             (135, 497),
    //             (14, 5),
    //             (46, 240),
    //             (103, 262),
    //             (69, 265),
    //             (80, 237),
    //             (76, 274),
    //             (73, 228),
    //             (100, 384),
    //             (49, 118),
    //             (126, 404),
    //             (23, 98),
    //             (149, 476),
    //             (0, 8),
    //             (0, 18),
    //         ],
    //         [
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (0, 0),
    //             (38, 123),
    //             (0, 56),
    //             (38, 67),
    //             (0, 82),
    //             (38, 41),
    //             (0, 108),
    //             (38, 15),
    //             (0, 121),
    //             (38, 2),
    //             (3, 123),
    //             (35, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (38, 123),
    //             (0, 0),
    //             (21, 70),
    //             (17, 53),
    //             (9, 29),
    //             (29, 94),
    //             (9, 47),
    //             (29, 76),
    //             (9, 59),
    //             (29, 64),
    //             (9, 77),
    //             (29, 46),
    //             (14, 94),
    //             (24, 29),
    //             (23, 98),
    //             (15, 25),
    //             (25, 112),
    //             (13, 11),
    //             (5, 24),
    //             (33, 99),
    //             (12, 35),
    //             (26, 88),
    //             (22, 78),
    //             (16, 45),
    //             (25, 79),
    //             (13, 44),
    //             (36, 95),
    //             (2, 28),
    //             (36, 97),
    //             (2, 26),
    //             (38, 123),
    //             (0, 0),
    //             (22, 70),
    //             (16, 53),
    //             (27, 79),
    //             (11, 44),
    //             (27, 86),
    //             (11, 37),
    //             (32, 101),
    //             (6, 22),
    //             (34, 103),
    //             (4, 20),
    //             (32, 113),
    //             (2, 1),
    //             (4, 9),
    //         ],
    //         [
    //             (72, 184),
    //             (51, 284),
    //             (0, 51),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 6),
    //             (146, 484),
    //             (3, 74),
    //             (147, 499),
    //             (2, 59),
    //             (149, 502),
    //             (0, 56),
    //             (149, 519),
    //             (0, 39),
    //             (149, 538),
    //             (0, 20),
    //             (149, 553),
    //             (0, 5),
    //             (149, 502),
    //             (0, 56),
    //             (149, 558),
    //             (0, 0),
    //             (149, 558),
    //             (0, 0),
    //             (149, 558),
    //             (0, 0),
    //             (149, 558),
    //             (0, 0),
    //             (149, 558),
    //             (0, 0),
    //             (131, 520),
    //             (18, 38),
    //             (133, 520),
    //             (16, 38),
    //             (133, 539),
    //             (16, 19),
    //             (133, 548),
    //             (16, 10),
    //             (137, 548),
    //             (12, 10),
    //             (137, 558),
    //             (12, 0),
    //             (148, 558),
    //             (1, 0),
    //             (46, 255),
    //             (103, 303),
    //             (3, 68),
    //             (146, 490),
    //             (22, 129),
    //             (127, 429),
    //             (22, 208),
    //             (127, 350),
    //             (23, 296),
    //             (126, 262),
    //             (69, 350),
    //             (80, 208),
    //             (103, 413),
    //             (46, 145),
    //             (148, 473),
    //             (1, 85),
    //             (24, 51),
    //             (125, 507),
    //             (43, 150),
    //             (106, 408),
    //             (89, 329),
    //             (60, 229),
    //             (104, 377),
    //             (45, 181),
    //             (114, 408),
    //             (35, 150),
    //             (119, 440),
    //             (30, 118),
    //             (135, 553),
    //             (14, 5),
    //             (46, 259),
    //             (103, 299),
    //             (69, 289),
    //             (80, 269),
    //             (76, 305),
    //             (73, 253),
    //             (100, 424),
    //             (49, 134),
    //             (126, 446),
    //             (23, 112),
    //             (149, 532),
    //             (0, 8),
    //             (0, 18),
    //         ],
    //         [
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 26),
    //             (38, 41),
    //             (0, 52),
    //             (38, 15),
    //             (0, 65),
    //             (38, 2),
    //             (3, 67),
    //             (35, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (38, 67),
    //             (0, 0),
    //             (21, 51),
    //             (17, 16),
    //             (9, 11),
    //             (29, 56),
    //             (9, 29),
    //             (29, 38),
    //             (9, 29),
    //             (29, 38),
    //             (9, 33),
    //             (29, 34),
    //             (14, 50),
    //             (24, 17),
    //             (23, 52),
    //             (15, 15),
    //             (25, 56),
    //             (13, 11),
    //             (5, 21),
    //             (33, 46),
    //             (12, 27),
    //             (26, 40),
    //             (22, 46),
    //             (16, 21),
    //             (25, 47),
    //             (13, 20),
    //             (36, 60),
    //             (2, 7),
    //             (36, 60),
    //             (2, 7),
    //             (38, 67),
    //             (0, 0),
    //             (22, 51),
    //             (16, 16),
    //             (27, 55),
    //             (11, 12),
    //             (27, 55),
    //             (11, 12),
    //             (32, 61),
    //             (6, 6),
    //             (34, 61),
    //             (4, 6),
    //             (32, 57),
    //             (2, 1),
    //             (4, 9),
    //         ],
    //         [
    //             (72, 184),
    //             (51, 310),
    //             (0, 51),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 6),
    //             (146, 510),
    //             (3, 74),
    //             (147, 525),
    //             (2, 59),
    //             (149, 528),
    //             (0, 56),
    //             (149, 545),
    //             (0, 39),
    //             (149, 564),
    //             (0, 20),
    //             (149, 579),
    //             (0, 5),
    //             (149, 502),
    //             (0, 82),
    //             (149, 558),
    //             (0, 26),
    //             (149, 584),
    //             (0, 0),
    //             (149, 584),
    //             (0, 0),
    //             (149, 584),
    //             (0, 0),
    //             (149, 584),
    //             (0, 0),
    //             (131, 546),
    //             (18, 38),
    //             (133, 546),
    //             (16, 38),
    //             (133, 565),
    //             (16, 19),
    //             (133, 574),
    //             (16, 10),
    //             (137, 574),
    //             (12, 10),
    //             (137, 584),
    //             (12, 0),
    //             (148, 584),
    //             (1, 0),
    //             (46, 273),
    //             (103, 311),
    //             (3, 76),
    //             (146, 508),
    //             (22, 149),
    //             (127, 435),
    //             (22, 228),
    //             (127, 356),
    //             (23, 317),
    //             (126, 267),
    //             (69, 372),
    //             (80, 212),
    //             (103, 436),
    //             (46, 148),
    //             (148, 498),
    //             (1, 86),
    //             (24, 54),
    //             (125, 530),
    //             (43, 154),
    //             (106, 430),
    //             (89, 343),
    //             (60, 241),
    //             (104, 391),
    //             (45, 193),
    //             (114, 432),
    //             (35, 152),
    //             (119, 464),
    //             (30, 120),
    //             (135, 579),
    //             (14, 5),
    //             (46, 277),
    //             (103, 307),
    //             (69, 307),
    //             (80, 277),
    //             (76, 323),
    //             (73, 261),
    //             (100, 446),
    //             (49, 138),
    //             (126, 468),
    //             (23, 116),
    //             (149, 558),
    //             (0, 8),
    //             (0, 18),
    //         ],
    //         [
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 26),
    //             (38, 15),
    //             (0, 39),
    //             (38, 2),
    //             (3, 41),
    //             (35, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (38, 41),
    //             (0, 0),
    //             (21, 33),
    //             (17, 8),
    //             (9, 3),
    //             (29, 38),
    //             (9, 9),
    //             (29, 32),
    //             (9, 9),
    //             (29, 32),
    //             (9, 12),
    //             (29, 29),
    //             (14, 28),
    //             (24, 13),
    //             (23, 29),
    //             (15, 12),
    //             (25, 31),
    //             (13, 10),
    //             (5, 18),
    //             (33, 23),
    //             (12, 23),
    //             (26, 18),
    //             (22, 32),
    //             (16, 9),
    //             (25, 33),
    //             (13, 8),
    //             (36, 36),
    //             (2, 5),
    //             (36, 36),
    //             (2, 5),
    //             (38, 41),
    //             (0, 0),
    //             (22, 33),
    //             (16, 8),
    //             (27, 37),
    //             (11, 4),
    //             (27, 37),
    //             (11, 4),
    //             (32, 39),
    //             (6, 2),
    //             (34, 39),
    //             (4, 2),
    //             (32, 31),
    //             (2, 1),
    //             (4, 9),
    //         ],
    //         [
    //             (72, 184),
    //             (51, 336),
    //             (0, 51),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 6),
    //             (146, 536),
    //             (3, 74),
    //             (147, 551),
    //             (2, 59),
    //             (149, 554),
    //             (0, 56),
    //             (149, 571),
    //             (0, 39),
    //             (149, 590),
    //             (0, 20),
    //             (149, 605),
    //             (0, 5),
    //             (149, 502),
    //             (0, 108),
    //             (149, 558),
    //             (0, 52),
    //             (149, 584),
    //             (0, 26),
    //             (149, 610),
    //             (0, 0),
    //             (149, 610),
    //             (0, 0),
    //             (149, 610),
    //             (0, 0),
    //             (131, 572),
    //             (18, 38),
    //             (133, 572),
    //             (16, 38),
    //             (133, 591),
    //             (16, 19),
    //             (133, 600),
    //             (16, 10),
    //             (137, 600),
    //             (12, 10),
    //             (137, 610),
    //             (12, 0),
    //             (148, 610),
    //             (1, 0),
    //             (46, 291),
    //             (103, 319),
    //             (3, 76),
    //             (146, 534),
    //             (22, 152),
    //             (127, 458),
    //             (22, 231),
    //             (127, 379),
    //             (23, 323),
    //             (126, 287),
    //             (69, 389),
    //             (80, 221),
    //             (103, 454),
    //             (46, 156),
    //             (148, 517),
    //             (1, 93),
    //             (24, 68),
    //             (125, 542),
    //             (43, 172),
    //             (106, 438),
    //             (89, 364),
    //             (60, 246),
    //             (104, 413),
    //             (45, 197),
    //             (114, 455),
    //             (35, 155),
    //             (119, 487),
    //             (30, 123),
    //             (135, 605),
    //             (14, 5),
    //             (46, 295),
    //             (103, 315),
    //             (69, 329),
    //             (80, 281),
    //             (76, 345),
    //             (73, 265),
    //             (100, 470),
    //             (49, 140),
    //             (126, 492),
    //             (23, 118),
    //             (149, 579),
    //             (0, 9),
    //             (0, 22),
    //         ],
    //         [
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 13),
    //             (38, 2),
    //             (3, 15),
    //             (35, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (38, 15),
    //             (0, 0),
    //             (21, 15),
    //             (17, 0),
    //             (9, 3),
    //             (29, 12),
    //             (9, 6),
    //             (29, 9),
    //             (9, 6),
    //             (29, 9),
    //             (9, 6),
    //             (29, 9),
    //             (14, 11),
    //             (24, 4),
    //             (23, 11),
    //             (15, 4),
    //             (25, 12),
    //             (13, 3),
    //             (5, 4),
    //             (33, 11),
    //             (12, 5),
    //             (26, 10),
    //             (22, 11),
    //             (16, 4),
    //             (25, 11),
    //             (13, 4),
    //             (36, 13),
    //             (2, 2),
    //             (36, 13),
    //             (2, 2),
    //             (38, 15),
    //             (0, 0),
    //             (22, 15),
    //             (16, 0),
    //             (27, 15),
    //             (11, 0),
    //             (27, 15),
    //             (11, 0),
    //             (32, 15),
    //             (6, 0),
    //             (34, 15),
    //             (4, 0),
    //             (32, 10),
    //             (2, 0),
    //             (4, 5),
    //         ],
    //         [
    //             (72, 184),
    //             (51, 349),
    //             (0, 51),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 6),
    //             (146, 549),
    //             (3, 74),
    //             (147, 564),
    //             (2, 59),
    //             (149, 567),
    //             (0, 56),
    //             (149, 584),
    //             (0, 39),
    //             (149, 603),
    //             (0, 20),
    //             (149, 618),
    //             (0, 5),
    //             (149, 502),
    //             (0, 121),
    //             (149, 558),
    //             (0, 65),
    //             (149, 584),
    //             (0, 39),
    //             (149, 610),
    //             (0, 13),
    //             (149, 623),
    //             (0, 0),
    //             (149, 623),
    //             (0, 0),
    //             (131, 585),
    //             (18, 38),
    //             (133, 585),
    //             (16, 38),
    //             (133, 604),
    //             (16, 19),
    //             (133, 613),
    //             (16, 10),
    //             (137, 613),
    //             (12, 10),
    //             (137, 623),
    //             (12, 0),
    //             (148, 623),
    //             (1, 0),
    //             (46, 304),
    //             (103, 319),
    //             (3, 77),
    //             (146, 546),
    //             (22, 156),
    //             (127, 467),
    //             (22, 235),
    //             (127, 388),
    //             (23, 327),
    //             (126, 296),
    //             (69, 398),
    //             (80, 225),
    //             (103, 463),
    //             (46, 160),
    //             (148, 527),
    //             (1, 96),
    //             (24, 70),
    //             (125, 553),
    //             (43, 175),
    //             (106, 448),
    //             (89, 373),
    //             (60, 250),
    //             (104, 422),
    //             (45, 201),
    //             (114, 466),
    //             (35, 157),
    //             (119, 498),
    //             (30, 125),
    //             (135, 618),
    //             (14, 5),
    //             (46, 308),
    //             (103, 315),
    //             (69, 342),
    //             (80, 281),
    //             (76, 358),
    //             (73, 265),
    //             (100, 483),
    //             (49, 140),
    //             (126, 505),
    //             (23, 118),
    //             (149, 587),
    //             (0, 9),
    //             (0, 27),
    //         ],
    //         [
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (3, 2),
    //             (35, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (38, 2),
    //             (0, 0),
    //             (21, 2),
    //             (17, 0),
    //             (9, 2),
    //             (29, 0),
    //             (9, 2),
    //             (29, 0),
    //             (9, 2),
    //             (29, 0),
    //             (9, 2),
    //             (29, 0),
    //             (14, 2),
    //             (24, 0),
    //             (23, 2),
    //             (15, 0),
    //             (25, 2),
    //             (13, 0),
    //             (5, 2),
    //             (33, 0),
    //             (12, 2),
    //             (26, 0),
    //             (22, 2),
    //             (16, 0),
    //             (25, 2),
    //             (13, 0),
    //             (36, 2),
    //             (2, 0),
    //             (36, 2),
    //             (2, 0),
    //             (38, 2),
    //             (0, 0),
    //             (22, 2),
    //             (16, 0),
    //             (27, 2),
    //             (11, 0),
    //             (27, 2),
    //             (11, 0),
    //             (32, 2),
    //             (6, 0),
    //             (34, 2),
    //             (4, 0),
    //             (32, 2),
    //             (2, 0),
    //             (4, 0),
    //         ],
    //         [
    //             (72, 184),
    //             (54, 351),
    //             (0, 51),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 6),
    //             (149, 551),
    //             (3, 74),
    //             (150, 566),
    //             (2, 59),
    //             (152, 569),
    //             (0, 56),
    //             (152, 586),
    //             (0, 39),
    //             (152, 605),
    //             (0, 20),
    //             (152, 620),
    //             (0, 5),
    //             (149, 502),
    //             (3, 123),
    //             (149, 558),
    //             (3, 67),
    //             (149, 584),
    //             (3, 41),
    //             (149, 610),
    //             (3, 15),
    //             (149, 623),
    //             (3, 2),
    //             (152, 625),
    //             (0, 0),
    //             (134, 587),
    //             (18, 38),
    //             (136, 587),
    //             (16, 38),
    //             (136, 606),
    //             (16, 19),
    //             (136, 615),
    //             (16, 10),
    //             (140, 615),
    //             (12, 10),
    //             (140, 625),
    //             (12, 0),
    //             (151, 625),
    //             (1, 0),
    //             (46, 306),
    //             (106, 319),
    //             (3, 79),
    //             (149, 546),
    //             (22, 158),
    //             (130, 467),
    //             (22, 237),
    //             (130, 388),
    //             (23, 329),
    //             (129, 296),
    //             (70, 400),
    //             (82, 225),
    //             (104, 465),
    //             (48, 160),
    //             (151, 529),
    //             (1, 96),
    //             (24, 72),
    //             (128, 553),
    //             (43, 177),
    //             (109, 448),
    //             (90, 375),
    //             (62, 250),
    //             (105, 424),
    //             (47, 201),
    //             (115, 468),
    //             (37, 157),
    //             (120, 500),
    //             (32, 125),
    //             (138, 620),
    //             (14, 5),
    //             (46, 310),
    //             (106, 315),
    //             (70, 344),
    //             (82, 281),
    //             (77, 360),
    //             (75, 265),
    //             (101, 485),
    //             (51, 140),
    //             (129, 507),
    //             (23, 118),
    //             (152, 589),
    //             (0, 9),
    //             (0, 27),
    //         ],
    //         [
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (21, 0),
    //             (14, 0),
    //             (9, 0),
    //             (26, 0),
    //             (9, 0),
    //             (26, 0),
    //             (9, 0),
    //             (26, 0),
    //             (9, 0),
    //             (26, 0),
    //             (13, 0),
    //             (22, 0),
    //             (22, 0),
    //             (13, 0),
    //             (22, 0),
    //             (13, 0),
    //             (5, 0),
    //             (30, 0),
    //             (12, 0),
    //             (23, 0),
    //             (21, 0),
    //             (14, 0),
    //             (24, 0),
    //             (11, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (35, 0),
    //             (0, 0),
    //             (22, 0),
    //             (13, 0),
    //             (26, 0),
    //             (9, 0),
    //             (26, 0),
    //             (9, 0),
    //             (31, 0),
    //             (4, 0),
    //             (31, 0),
    //             (4, 0),
    //             (29, 0),
    //             (2, 0),
    //             (4, 0),
    //         ],
    //         [
    //             (70, 179),
    //             (89, 346),
    //             (0, 51),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (10, 6),
    //             (166, 513),
    //             (3, 74),
    //             (167, 528),
    //             (2, 59),
    //             (169, 531),
    //             (0, 56),
    //             (169, 548),
    //             (0, 39),
    //             (169, 567),
    //             (0, 20),
    //             (169, 582),
    //             (0, 5),
    //             (131, 464),
    //             (38, 123),
    //             (131, 520),
    //             (38, 67),
    //             (131, 546),
    //             (38, 41),
    //             (131, 572),
    //             (38, 15),
    //             (131, 585),
    //             (38, 2),
    //             (134, 587),
    //             (35, 0),
    //             (169, 587),
    //             (0, 0),
    //             (169, 587),
    //             (0, 0),
    //             (169, 587),
    //             (0, 0),
    //             (169, 587),
    //             (0, 0),
    //             (169, 587),
    //             (0, 0),
    //             (169, 587),
    //             (0, 0),
    //             (169, 587),
    //             (0, 0),
    //             (56, 282),
    //             (113, 305),
    //             (12, 79),
    //             (157, 508),
    //             (31, 158),
    //             (138, 429),
    //             (31, 237),
    //             (138, 350),
    //             (32, 329),
    //             (137, 258),
    //             (72, 391),
    //             (97, 196),
    //             (111, 450),
    //             (58, 137),
    //             (155, 500),
    //             (14, 87),
    //             (26, 71),
    //             (143, 516),
    //             (50, 170),
    //             (119, 417),
    //             (104, 356),
    //             (65, 231),
    //             (112, 405),
    //             (57, 182),
    //             (133, 448),
    //             (36, 139),
    //             (138, 469),
    //             (31, 118),
    //             (155, 583),
    //             (14, 4),
    //             (57, 286),
    //             (112, 301),
    //             (85, 320),
    //             (84, 267),
    //             (92, 336),
    //             (77, 251),
    //             (121, 452),
    //             (48, 135),
    //             (143, 474),
    //             (26, 113),
    //             (163, 555),
    //             (2, 7),
    //             (4, 25),
    //         ],
    //         [
    //             (2, 5),
    //             (0, 5),
    //             (0, 0),
    //             (11, 9),
    //             (5, 0),
    //             (0, 19),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (0, 0),
    //             (18, 38),
    //             (2, 0),
    //             (16, 38),
    //             (2, 19),
    //             (16, 19),
    //             (2, 28),
    //             (16, 10),
    //             (6, 28),
    //             (12, 10),
    //             (6, 38),
    //             (12, 0),
    //             (17, 38),
    //             (1, 0),
    //             (11, 24),
    //             (7, 14),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (0, 0),
    //             (18, 38),
    //             (11, 9),
    //             (7, 29),
    //             (15, 15),
    //             (3, 23),
    //             (18, 29),
    //             (0, 9),
    //             (3, 1),
    //             (15, 37),
    //             (5, 7),
    //             (13, 31),
    //             (7, 19),
    //             (11, 19),
    //             (17, 19),
    //             (1, 19),
    //             (17, 20),
    //             (1, 18),
    //             (17, 31),
    //             (1, 7),
    //             (18, 37),
    //             (0, 1),
    //             (11, 24),
    //             (7, 14),
    //             (11, 24),
    //             (7, 14),
    //             (11, 24),
    //             (7, 14),
    //             (11, 33),
    //             (7, 5),
    //             (17, 33),
    //             (1, 5),
    //             (18, 34),
    //             (0, 2),
    //             (0, 2),
    //         ],
    //         [
    //             (72, 179),
    //             (89, 346),
    //             (0, 51),
    //             (0, 0),
    //             (0, 5),
    //             (0, 0),
    //             (10, 6),
    //             (168, 513),
    //             (3, 74),
    //             (169, 528),
    //             (2, 59),
    //             (171, 531),
    //             (0, 56),
    //             (171, 548),
    //             (0, 39),
    //             (171, 567),
    //             (0, 20),
    //             (171, 582),
    //             (0, 5),
    //             (133, 464),
    //             (38, 123),
    //             (133, 520),
    //             (38, 67),
    //             (133, 546),
    //             (38, 41),
    //             (133, 572),
    //             (38, 15),
    //             (133, 585),
    //             (38, 2),
    //             (136, 587),
    //             (35, 0),
    //             (169, 587),
    //             (2, 0),
    //             (171, 587),
    //             (0, 0),
    //             (171, 587),
    //             (0, 0),
    //             (171, 587),
    //             (0, 0),
    //             (171, 587),
    //             (0, 0),
    //             (171, 587),
    //             (0, 0),
    //             (171, 587),
    //             (0, 0),
    //             (58, 282),
    //             (113, 305),
    //             (12, 79),
    //             (159, 508),
    //             (31, 158),
    //             (140, 429),
    //             (31, 237),
    //             (140, 350),
    //             (32, 329),
    //             (139, 258),
    //             (74, 391),
    //             (97, 196),
    //             (113, 450),
    //             (58, 137),
    //             (157, 500),
    //             (14, 87),
    //             (26, 71),
    //             (145, 516),
    //             (50, 170),
    //             (121, 417),
    //             (104, 356),
    //             (67, 231),
    //             (114, 405),
    //             (57, 182),
    //             (135, 448),
    //             (36, 139),
    //             (140, 469),
    //             (31, 118),
    //             (157, 583),
    //             (14, 4),
    //             (59, 286),
    //             (112, 301),
    //             (87, 320),
    //             (84, 267),
    //             (94, 336),
    //             (77, 251),
    //             (123, 452),
    //             (48, 135),
    //             (145, 474),
    //             (26, 113),
    //             (165, 555),
    //             (2, 7),
    //             (4, 25),
    //         ],
    //         [
    //             (0, 5),
    //             (0, 5),
    //             (0, 0),
    //             (11, 9),
    //             (5, 0),
    //             (0, 19),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 19),
    //             (16, 19),
    //             (0, 28),
    //             (16, 10),
    //             (4, 28),
    //             (12, 10),
    //             (4, 38),
    //             (12, 0),
    //             (15, 38),
    //             (1, 0),
    //             (9, 24),
    //             (7, 14),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (0, 0),
    //             (16, 38),
    //             (9, 9),
    //             (7, 29),
    //             (13, 15),
    //             (3, 23),
    //             (16, 29),
    //             (0, 9),
    //             (3, 1),
    //             (13, 37),
    //             (5, 7),
    //             (11, 31),
    //             (7, 19),
    //             (9, 19),
    //             (15, 19),
    //             (1, 19),
    //             (15, 20),
    //             (1, 18),
    //             (15, 31),
    //             (1, 7),
    //             (16, 37),
    //             (0, 1),
    //             (9, 24),
    //             (7, 14),
    //             (9, 24),
    //             (7, 14),
    //             (9, 24),
    //             (7, 14),
    //             (9, 33),
    //             (7, 5),
    //             (15, 33),
    //             (1, 5),
    //             (16, 34),
    //             (0, 2),
    //             (0, 2),
    //         ],
    //         [
    //             (72, 179),
    //             (89, 346),
    //             (0, 51),
    //             (0, 0),
    //             (0, 5),
    //             (0, 19),
    //             (10, 6),
    //             (168, 532),
    //             (3, 74),
    //             (169, 547),
    //             (2, 59),
    //             (171, 550),
    //             (0, 56),
    //             (171, 567),
    //             (0, 39),
    //             (171, 586),
    //             (0, 20),
    //             (171, 601),
    //             (0, 5),
    //             (133, 483),
    //             (38, 123),
    //             (133, 539),
    //             (38, 67),
    //             (133, 565),
    //             (38, 41),
    //             (133, 591),
    //             (38, 15),
    //             (133, 604),
    //             (38, 2),
    //             (136, 606),
    //             (35, 0),
    //             (169, 587),
    //             (2, 19),
    //             (171, 587),
    //             (0, 19),
    //             (171, 606),
    //             (0, 0),
    //             (171, 606),
    //             (0, 0),
    //             (171, 606),
    //             (0, 0),
    //             (171, 606),
    //             (0, 0),
    //             (171, 606),
    //             (0, 0),
    //             (58, 289),
    //             (113, 317),
    //             (12, 79),
    //             (159, 527),
    //             (31, 158),
    //             (140, 448),
    //             (31, 237),
    //             (140, 369),
    //             (32, 329),
    //             (139, 277),
    //             (74, 395),
    //             (97, 211),
    //             (113, 457),
    //             (58, 149),
    //             (157, 510),
    //             (14, 96),
    //             (26, 71),
    //             (145, 535),
    //             (50, 172),
    //             (121, 434),
    //             (104, 365),
    //             (67, 241),
    //             (114, 414),
    //             (57, 192),
    //             (135, 457),
    //             (36, 149),
    //             (140, 488),
    //             (31, 118),
    //             (157, 602),
    //             (14, 4),
    //             (59, 293),
    //             (112, 313),
    //             (87, 327),
    //             (84, 279),
    //             (94, 343),
    //             (77, 263),
    //             (123, 467),
    //             (48, 139),
    //             (145, 489),
    //             (26, 117),
    //             (165, 570),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (0, 5),
    //             (0, 5),
    //             (0, 0),
    //             (11, 9),
    //             (5, 0),
    //             (0, 0),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 9),
    //             (16, 10),
    //             (4, 9),
    //             (12, 10),
    //             (4, 19),
    //             (12, 0),
    //             (15, 19),
    //             (1, 0),
    //             (9, 17),
    //             (7, 2),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (0, 0),
    //             (16, 19),
    //             (9, 5),
    //             (7, 14),
    //             (13, 8),
    //             (3, 11),
    //             (16, 19),
    //             (0, 0),
    //             (3, 1),
    //             (13, 18),
    //             (5, 5),
    //             (11, 14),
    //             (7, 10),
    //             (9, 9),
    //             (15, 10),
    //             (1, 9),
    //             (15, 11),
    //             (1, 8),
    //             (15, 12),
    //             (1, 7),
    //             (16, 18),
    //             (0, 1),
    //             (9, 17),
    //             (7, 2),
    //             (9, 17),
    //             (7, 2),
    //             (9, 17),
    //             (7, 2),
    //             (9, 18),
    //             (7, 1),
    //             (15, 18),
    //             (1, 1),
    //             (16, 19),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (72, 179),
    //             (89, 346),
    //             (0, 51),
    //             (0, 9),
    //             (0, 5),
    //             (0, 19),
    //             (10, 6),
    //             (168, 541),
    //             (3, 74),
    //             (169, 556),
    //             (2, 59),
    //             (171, 559),
    //             (0, 56),
    //             (171, 576),
    //             (0, 39),
    //             (171, 595),
    //             (0, 20),
    //             (171, 610),
    //             (0, 5),
    //             (133, 492),
    //             (38, 123),
    //             (133, 548),
    //             (38, 67),
    //             (133, 574),
    //             (38, 41),
    //             (133, 600),
    //             (38, 15),
    //             (133, 613),
    //             (38, 2),
    //             (136, 615),
    //             (35, 0),
    //             (169, 587),
    //             (2, 28),
    //             (171, 587),
    //             (0, 28),
    //             (171, 606),
    //             (0, 9),
    //             (171, 615),
    //             (0, 0),
    //             (171, 615),
    //             (0, 0),
    //             (171, 615),
    //             (0, 0),
    //             (171, 615),
    //             (0, 0),
    //             (58, 298),
    //             (113, 317),
    //             (12, 79),
    //             (159, 536),
    //             (31, 158),
    //             (140, 457),
    //             (31, 237),
    //             (140, 378),
    //             (32, 329),
    //             (139, 286),
    //             (74, 397),
    //             (97, 218),
    //             (113, 462),
    //             (58, 153),
    //             (157, 519),
    //             (14, 96),
    //             (26, 72),
    //             (145, 543),
    //             (50, 176),
    //             (121, 439),
    //             (104, 371),
    //             (67, 244),
    //             (114, 420),
    //             (57, 195),
    //             (135, 463),
    //             (36, 152),
    //             (140, 495),
    //             (31, 120),
    //             (157, 611),
    //             (14, 4),
    //             (59, 302),
    //             (112, 313),
    //             (87, 336),
    //             (84, 279),
    //             (94, 352),
    //             (77, 263),
    //             (123, 476),
    //             (48, 139),
    //             (145, 498),
    //             (26, 117),
    //             (165, 579),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (0, 5),
    //             (0, 5),
    //             (0, 0),
    //             (11, 0),
    //             (5, 0),
    //             (0, 0),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (4, 0),
    //             (12, 10),
    //             (4, 10),
    //             (12, 0),
    //             (15, 10),
    //             (1, 0),
    //             (9, 8),
    //             (7, 2),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (0, 0),
    //             (16, 10),
    //             (9, 3),
    //             (7, 7),
    //             (13, 3),
    //             (3, 7),
    //             (16, 10),
    //             (0, 0),
    //             (3, 0),
    //             (13, 10),
    //             (5, 1),
    //             (11, 9),
    //             (7, 4),
    //             (9, 6),
    //             (15, 4),
    //             (1, 6),
    //             (15, 5),
    //             (1, 5),
    //             (15, 5),
    //             (1, 5),
    //             (16, 9),
    //             (0, 1),
    //             (9, 8),
    //             (7, 2),
    //             (9, 8),
    //             (7, 2),
    //             (9, 8),
    //             (7, 2),
    //             (9, 9),
    //             (7, 1),
    //             (15, 9),
    //             (1, 1),
    //             (16, 10),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (72, 179),
    //             (89, 346),
    //             (0, 51),
    //             (0, 9),
    //             (4, 5),
    //             (0, 19),
    //             (10, 6),
    //             (172, 541),
    //             (3, 74),
    //             (173, 556),
    //             (2, 59),
    //             (175, 559),
    //             (0, 56),
    //             (175, 576),
    //             (0, 39),
    //             (175, 595),
    //             (0, 20),
    //             (175, 610),
    //             (0, 5),
    //             (137, 492),
    //             (38, 123),
    //             (137, 548),
    //             (38, 67),
    //             (137, 574),
    //             (38, 41),
    //             (137, 600),
    //             (38, 15),
    //             (137, 613),
    //             (38, 2),
    //             (140, 615),
    //             (35, 0),
    //             (169, 587),
    //             (6, 28),
    //             (171, 587),
    //             (4, 28),
    //             (171, 606),
    //             (4, 9),
    //             (171, 615),
    //             (4, 0),
    //             (175, 615),
    //             (0, 0),
    //             (175, 615),
    //             (0, 0),
    //             (175, 615),
    //             (0, 0),
    //             (62, 298),
    //             (113, 317),
    //             (12, 79),
    //             (163, 536),
    //             (31, 158),
    //             (144, 457),
    //             (31, 237),
    //             (144, 378),
    //             (32, 329),
    //             (143, 286),
    //             (78, 397),
    //             (97, 218),
    //             (117, 462),
    //             (58, 153),
    //             (161, 519),
    //             (14, 96),
    //             (29, 72),
    //             (146, 543),
    //             (54, 176),
    //             (121, 439),
    //             (108, 371),
    //             (67, 244),
    //             (118, 420),
    //             (57, 195),
    //             (139, 463),
    //             (36, 152),
    //             (144, 495),
    //             (31, 120),
    //             (161, 611),
    //             (14, 4),
    //             (63, 302),
    //             (112, 313),
    //             (91, 336),
    //             (84, 279),
    //             (98, 352),
    //             (77, 263),
    //             (127, 476),
    //             (48, 139),
    //             (149, 498),
    //             (26, 117),
    //             (169, 579),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (0, 5),
    //             (0, 5),
    //             (0, 0),
    //             (11, 0),
    //             (1, 0),
    //             (0, 0),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 10),
    //             (12, 0),
    //             (11, 10),
    //             (1, 0),
    //             (5, 8),
    //             (7, 2),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (0, 0),
    //             (12, 10),
    //             (5, 3),
    //             (7, 7),
    //             (9, 3),
    //             (3, 7),
    //             (12, 10),
    //             (0, 0),
    //             (0, 0),
    //             (12, 10),
    //             (1, 1),
    //             (11, 9),
    //             (3, 4),
    //             (9, 6),
    //             (11, 4),
    //             (1, 6),
    //             (11, 5),
    //             (1, 5),
    //             (11, 5),
    //             (1, 5),
    //             (12, 9),
    //             (0, 1),
    //             (5, 8),
    //             (7, 2),
    //             (5, 8),
    //             (7, 2),
    //             (5, 8),
    //             (7, 2),
    //             (5, 9),
    //             (7, 1),
    //             (11, 9),
    //             (1, 1),
    //             (12, 10),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (72, 184),
    //             (89, 351),
    //             (0, 51),
    //             (0, 9),
    //             (4, 5),
    //             (0, 19),
    //             (10, 6),
    //             (172, 551),
    //             (3, 74),
    //             (173, 566),
    //             (2, 59),
    //             (175, 569),
    //             (0, 56),
    //             (175, 586),
    //             (0, 39),
    //             (175, 605),
    //             (0, 20),
    //             (175, 620),
    //             (0, 5),
    //             (137, 502),
    //             (38, 123),
    //             (137, 558),
    //             (38, 67),
    //             (137, 584),
    //             (38, 41),
    //             (137, 610),
    //             (38, 15),
    //             (137, 623),
    //             (38, 2),
    //             (140, 625),
    //             (35, 0),
    //             (169, 587),
    //             (6, 38),
    //             (171, 587),
    //             (4, 38),
    //             (171, 606),
    //             (4, 19),
    //             (171, 615),
    //             (4, 10),
    //             (175, 615),
    //             (0, 10),
    //             (175, 625),
    //             (0, 0),
    //             (175, 625),
    //             (0, 0),
    //             (62, 306),
    //             (113, 319),
    //             (12, 79),
    //             (163, 546),
    //             (31, 158),
    //             (144, 467),
    //             (31, 237),
    //             (144, 388),
    //             (32, 329),
    //             (143, 296),
    //             (78, 400),
    //             (97, 225),
    //             (117, 465),
    //             (58, 160),
    //             (161, 529),
    //             (14, 96),
    //             (29, 72),
    //             (146, 553),
    //             (54, 177),
    //             (121, 448),
    //             (108, 375),
    //             (67, 250),
    //             (118, 424),
    //             (57, 201),
    //             (139, 468),
    //             (36, 157),
    //             (144, 500),
    //             (31, 125),
    //             (161, 620),
    //             (14, 5),
    //             (63, 310),
    //             (112, 315),
    //             (91, 344),
    //             (84, 281),
    //             (98, 360),
    //             (77, 265),
    //             (127, 485),
    //             (48, 140),
    //             (149, 507),
    //             (26, 118),
    //             (169, 589),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (11, 0),
    //             (1, 0),
    //             (0, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (11, 0),
    //             (1, 0),
    //             (5, 0),
    //             (7, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (0, 0),
    //             (12, 0),
    //             (5, 0),
    //             (7, 0),
    //             (9, 0),
    //             (3, 0),
    //             (12, 0),
    //             (0, 0),
    //             (0, 0),
    //             (12, 0),
    //             (1, 0),
    //             (11, 0),
    //             (3, 0),
    //             (9, 0),
    //             (11, 0),
    //             (1, 0),
    //             (11, 0),
    //             (1, 0),
    //             (11, 0),
    //             (1, 0),
    //             (12, 0),
    //             (0, 0),
    //             (5, 0),
    //             (7, 0),
    //             (5, 0),
    //             (7, 0),
    //             (5, 0),
    //             (7, 0),
    //             (5, 0),
    //             (7, 0),
    //             (11, 0),
    //             (1, 0),
    //             (12, 0),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (72, 184),
    //             (89, 351),
    //             (0, 51),
    //             (11, 9),
    //             (4, 5),
    //             (0, 19),
    //             (10, 6),
    //             (183, 551),
    //             (3, 74),
    //             (184, 566),
    //             (2, 59),
    //             (186, 569),
    //             (0, 56),
    //             (186, 586),
    //             (0, 39),
    //             (186, 605),
    //             (0, 20),
    //             (186, 620),
    //             (0, 5),
    //             (148, 502),
    //             (38, 123),
    //             (148, 558),
    //             (38, 67),
    //             (148, 584),
    //             (38, 41),
    //             (148, 610),
    //             (38, 15),
    //             (148, 623),
    //             (38, 2),
    //             (151, 625),
    //             (35, 0),
    //             (169, 587),
    //             (17, 38),
    //             (171, 587),
    //             (15, 38),
    //             (171, 606),
    //             (15, 19),
    //             (171, 615),
    //             (15, 10),
    //             (175, 615),
    //             (11, 10),
    //             (175, 625),
    //             (11, 0),
    //             (186, 625),
    //             (0, 0),
    //             (66, 306),
    //             (120, 319),
    //             (12, 79),
    //             (174, 546),
    //             (31, 158),
    //             (155, 467),
    //             (31, 237),
    //             (155, 388),
    //             (32, 329),
    //             (154, 296),
    //             (82, 400),
    //             (104, 225),
    //             (125, 465),
    //             (61, 160),
    //             (172, 529),
    //             (14, 96),
    //             (29, 72),
    //             (157, 553),
    //             (54, 177),
    //             (132, 448),
    //             (110, 375),
    //             (76, 250),
    //             (128, 424),
    //             (58, 201),
    //             (149, 468),
    //             (37, 157),
    //             (154, 500),
    //             (32, 125),
    //             (172, 620),
    //             (14, 5),
    //             (67, 310),
    //             (119, 315),
    //             (95, 344),
    //             (91, 281),
    //             (102, 360),
    //             (84, 265),
    //             (131, 485),
    //             (55, 140),
    //             (159, 507),
    //             (27, 118),
    //             (180, 589),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (1, 0),
    //             (0, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (0, 0),
    //             (1, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (1, 0),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (12, 70),
    //             (43, 165),
    //             (0, 51),
    //             (4, 9),
    //             (5, 1),
    //             (0, 7),
    //             (3, 3),
    //             (64, 243),
    //             (3, 63),
    //             (65, 251),
    //             (2, 55),
    //             (67, 254),
    //             (0, 52),
    //             (67, 271),
    //             (0, 35),
    //             (67, 290),
    //             (0, 16),
    //             (67, 305),
    //             (0, 1),
    //             (46, 236),
    //             (21, 70),
    //             (46, 255),
    //             (21, 51),
    //             (46, 273),
    //             (21, 33),
    //             (46, 291),
    //             (21, 15),
    //             (46, 304),
    //             (21, 2),
    //             (46, 306),
    //             (21, 0),
    //             (56, 282),
    //             (11, 24),
    //             (58, 282),
    //             (9, 24),
    //             (58, 289),
    //             (9, 17),
    //             (58, 298),
    //             (9, 8),
    //             (62, 298),
    //             (5, 8),
    //             (62, 306),
    //             (5, 0),
    //             (66, 306),
    //             (1, 0),
    //             (67, 306),
    //             (0, 0),
    //             (4, 38),
    //             (63, 268),
    //             (11, 69),
    //             (56, 237),
    //             (11, 100),
    //             (56, 206),
    //             (12, 133),
    //             (55, 173),
    //             (27, 172),
    //             (40, 134),
    //             (41, 204),
    //             (26, 102),
    //             (54, 241),
    //             (13, 65),
    //             (17, 46),
    //             (50, 260),
    //             (35, 91),
    //             (32, 215),
    //             (49, 182),
    //             (18, 124),
    //             (58, 230),
    //             (9, 76),
    //             (63, 247),
    //             (4, 59),
    //             (63, 253),
    //             (4, 53),
    //             (65, 305),
    //             (2, 1),
    //             (67, 306),
    //             (0, 0),
    //             (67, 306),
    //             (0, 0),
    //             (67, 306),
    //             (0, 0),
    //             (67, 306),
    //             (0, 0),
    //             (67, 306),
    //             (0, 0),
    //             (61, 270),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (60, 114),
    //             (46, 186),
    //             (0, 0),
    //             (7, 0),
    //             (0, 4),
    //             (0, 12),
    //             (7, 3),
    //             (120, 308),
    //             (0, 11),
    //             (120, 315),
    //             (0, 4),
    //             (120, 315),
    //             (0, 4),
    //             (120, 315),
    //             (0, 4),
    //             (120, 315),
    //             (0, 4),
    //             (120, 315),
    //             (0, 4),
    //             (103, 266),
    //             (17, 53),
    //             (103, 303),
    //             (17, 16),
    //             (103, 311),
    //             (17, 8),
    //             (103, 319),
    //             (17, 0),
    //             (103, 319),
    //             (17, 0),
    //             (106, 319),
    //             (14, 0),
    //             (113, 305),
    //             (7, 14),
    //             (113, 305),
    //             (7, 14),
    //             (113, 317),
    //             (7, 2),
    //             (113, 317),
    //             (7, 2),
    //             (113, 317),
    //             (7, 2),
    //             (113, 319),
    //             (7, 0),
    //             (120, 319),
    //             (0, 0),
    //             (0, 0),
    //             (120, 319),
    //             (8, 41),
    //             (112, 278),
    //             (20, 89),
    //             (100, 230),
    //             (20, 137),
    //             (100, 182),
    //             (20, 196),
    //             (100, 123),
    //             (56, 228),
    //             (64, 91),
    //             (85, 261),
    //             (35, 58),
    //             (119, 288),
    //             (1, 31),
    //             (12, 26),
    //             (108, 293),
    //             (20, 86),
    //             (100, 233),
    //             (62, 193),
    //             (58, 126),
    //             (71, 194),
    //             (49, 125),
    //             (87, 221),
    //             (33, 98),
    //             (92, 247),
    //             (28, 72),
    //             (108, 315),
    //             (12, 4),
    //             (1, 4),
    //             (119, 315),
    //             (29, 38),
    //             (91, 281),
    //             (36, 54),
    //             (84, 265),
    //             (65, 179),
    //             (55, 140),
    //             (93, 201),
    //             (27, 118),
    //             (120, 319),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (0, 22),
    //             (12, 57),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (11, 75),
    //             (1, 4),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (3, 50),
    //             (9, 29),
    //             (3, 68),
    //             (9, 11),
    //             (3, 76),
    //             (9, 3),
    //             (3, 76),
    //             (9, 3),
    //             (3, 77),
    //             (9, 2),
    //             (3, 79),
    //             (9, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (4, 38),
    //             (8, 41),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (12, 79),
    //             (0, 0),
    //             (1, 11),
    //             (11, 68),
    //             (3, 20),
    //             (9, 59),
    //             (6, 50),
    //             (6, 29),
    //             (6, 50),
    //             (6, 29),
    //             (12, 58),
    //             (0, 21),
    //             (12, 58),
    //             (0, 21),
    //             (12, 79),
    //             (0, 0),
    //             (5, 38),
    //             (7, 41),
    //             (6, 43),
    //             (6, 36),
    //             (6, 48),
    //             (6, 31),
    //             (10, 64),
    //             (2, 15),
    //             (10, 65),
    //             (2, 14),
    //             (12, 79),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (72, 162),
    //             (77, 294),
    //             (0, 51),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 6),
    //             (173, 476),
    //             (2, 70),
    //             (173, 487),
    //             (2, 59),
    //             (175, 490),
    //             (0, 56),
    //             (175, 507),
    //             (0, 39),
    //             (175, 526),
    //             (0, 20),
    //             (175, 541),
    //             (0, 5),
    //             (146, 452),
    //             (29, 94),
    //             (146, 490),
    //             (29, 56),
    //             (146, 508),
    //             (29, 38),
    //             (146, 534),
    //             (29, 12),
    //             (146, 546),
    //             (29, 0),
    //             (149, 546),
    //             (26, 0),
    //             (157, 508),
    //             (18, 38),
    //             (159, 508),
    //             (16, 38),
    //             (159, 527),
    //             (16, 19),
    //             (159, 536),
    //             (16, 10),
    //             (163, 536),
    //             (12, 10),
    //             (163, 546),
    //             (12, 0),
    //             (174, 546),
    //             (1, 0),
    //             (63, 268),
    //             (112, 278),
    //             (0, 0),
    //             (175, 546),
    //             (19, 79),
    //             (156, 467),
    //             (19, 158),
    //             (156, 388),
    //             (20, 250),
    //             (155, 296),
    //             (71, 321),
    //             (104, 225),
    //             (114, 386),
    //             (61, 160),
    //             (161, 450),
    //             (14, 96),
    //             (28, 61),
    //             (147, 485),
    //             (52, 157),
    //             (123, 389),
    //             (105, 325),
    //             (70, 221),
    //             (123, 374),
    //             (52, 172),
    //             (138, 410),
    //             (37, 136),
    //             (143, 442),
    //             (32, 104),
    //             (161, 541),
    //             (14, 5),
    //             (63, 272),
    //             (112, 274),
    //             (90, 301),
    //             (85, 245),
    //             (97, 312),
    //             (78, 234),
    //             (122, 421),
    //             (53, 125),
    //             (150, 442),
    //             (25, 104),
    //             (169, 510),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (0, 39),
    //             (31, 114),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (30, 147),
    //             (1, 11),
    //             (31, 153),
    //             (0, 5),
    //             (31, 153),
    //             (0, 5),
    //             (31, 157),
    //             (0, 1),
    //             (31, 157),
    //             (0, 1),
    //             (31, 158),
    //             (0, 0),
    //             (22, 111),
    //             (9, 47),
    //             (22, 129),
    //             (9, 29),
    //             (22, 149),
    //             (9, 9),
    //             (22, 152),
    //             (9, 6),
    //             (22, 156),
    //             (9, 2),
    //             (22, 158),
    //             (9, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (11, 69),
    //             (20, 89),
    //             (12, 79),
    //             (19, 79),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (31, 158),
    //             (0, 0),
    //             (7, 18),
    //             (24, 140),
    //             (13, 46),
    //             (18, 112),
    //             (22, 107),
    //             (9, 51),
    //             (22, 110),
    //             (9, 48),
    //             (31, 128),
    //             (0, 30),
    //             (31, 128),
    //             (0, 30),
    //             (31, 158),
    //             (0, 0),
    //             (12, 71),
    //             (19, 87),
    //             (18, 81),
    //             (13, 77),
    //             (20, 89),
    //             (11, 69),
    //             (25, 120),
    //             (6, 38),
    //             (26, 122),
    //             (5, 36),
    //             (31, 157),
    //             (0, 0),
    //             (0, 1),
    //         ],
    //         [
    //             (72, 145),
    //             (58, 237),
    //             (0, 46),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 6),
    //             (154, 404),
    //             (2, 63),
    //             (154, 413),
    //             (2, 54),
    //             (156, 416),
    //             (0, 51),
    //             (156, 429),
    //             (0, 38),
    //             (156, 448),
    //             (0, 19),
    //             (156, 462),
    //             (0, 5),
    //             (127, 391),
    //             (29, 76),
    //             (127, 429),
    //             (29, 38),
    //             (127, 435),
    //             (29, 32),
    //             (127, 458),
    //             (29, 9),
    //             (127, 467),
    //             (29, 0),
    //             (130, 467),
    //             (26, 0),
    //             (138, 429),
    //             (18, 38),
    //             (140, 429),
    //             (16, 38),
    //             (140, 448),
    //             (16, 19),
    //             (140, 457),
    //             (16, 10),
    //             (144, 457),
    //             (12, 10),
    //             (144, 467),
    //             (12, 0),
    //             (155, 467),
    //             (1, 0),
    //             (56, 237),
    //             (100, 230),
    //             (0, 0),
    //             (156, 467),
    //             (0, 0),
    //             (156, 467),
    //             (0, 79),
    //             (156, 388),
    //             (1, 171),
    //             (155, 296),
    //             (52, 242),
    //             (104, 225),
    //             (95, 307),
    //             (61, 160),
    //             (142, 371),
    //             (14, 96),
    //             (22, 54),
    //             (134, 413),
    //             (42, 131),
    //             (114, 336),
    //             (89, 268),
    //             (67, 199),
    //             (107, 314),
    //             (49, 153),
    //             (119, 340),
    //             (37, 127),
    //             (124, 372),
    //             (32, 95),
    //             (142, 462),
    //             (14, 5),
    //             (56, 239),
    //             (100, 228),
    //             (78, 263),
    //             (78, 204),
    //             (83, 271),
    //             (73, 196),
    //             (107, 365),
    //             (49, 102),
    //             (134, 385),
    //             (22, 82),
    //             (150, 432),
    //             (2, 9),
    //             (4, 26),
    //         ],
    //         [
    //             (0, 69),
    //             (31, 162),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 1),
    //             (30, 225),
    //             (1, 12),
    //             (31, 231),
    //             (0, 6),
    //             (31, 232),
    //             (0, 5),
    //             (31, 236),
    //             (0, 1),
    //             (31, 236),
    //             (0, 1),
    //             (31, 237),
    //             (0, 0),
    //             (22, 178),
    //             (9, 59),
    //             (22, 208),
    //             (9, 29),
    //             (22, 228),
    //             (9, 9),
    //             (22, 231),
    //             (9, 6),
    //             (22, 235),
    //             (9, 2),
    //             (22, 237),
    //             (9, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (11, 100),
    //             (20, 137),
    //             (12, 79),
    //             (19, 158),
    //             (31, 158),
    //             (0, 79),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (31, 237),
    //             (0, 0),
    //             (7, 24),
    //             (24, 213),
    //             (13, 70),
    //             (18, 167),
    //             (22, 155),
    //             (9, 82),
    //             (22, 158),
    //             (9, 79),
    //             (31, 180),
    //             (0, 57),
    //             (31, 180),
    //             (0, 57),
    //             (31, 237),
    //             (0, 0),
    //             (12, 104),
    //             (19, 133),
    //             (18, 120),
    //             (13, 117),
    //             (20, 136),
    //             (11, 101),
    //             (25, 180),
    //             (6, 57),
    //             (26, 182),
    //             (5, 55),
    //             (31, 236),
    //             (0, 0),
    //             (0, 1),
    //         ],
    //         [
    //             (72, 115),
    //             (58, 189),
    //             (0, 46),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 5),
    //             (154, 326),
    //             (2, 62),
    //             (154, 335),
    //             (2, 53),
    //             (156, 337),
    //             (0, 51),
    //             (156, 350),
    //             (0, 38),
    //             (156, 369),
    //             (0, 19),
    //             (156, 383),
    //             (0, 5),
    //             (127, 324),
    //             (29, 64),
    //             (127, 350),
    //             (29, 38),
    //             (127, 356),
    //             (29, 32),
    //             (127, 379),
    //             (29, 9),
    //             (127, 388),
    //             (29, 0),
    //             (130, 388),
    //             (26, 0),
    //             (138, 350),
    //             (18, 38),
    //             (140, 350),
    //             (16, 38),
    //             (140, 369),
    //             (16, 19),
    //             (140, 378),
    //             (16, 10),
    //             (144, 378),
    //             (12, 10),
    //             (144, 388),
    //             (12, 0),
    //             (155, 388),
    //             (1, 0),
    //             (56, 206),
    //             (100, 182),
    //             (0, 0),
    //             (156, 388),
    //             (0, 0),
    //             (156, 388),
    //             (0, 0),
    //             (156, 388),
    //             (1, 92),
    //             (155, 296),
    //             (52, 163),
    //             (104, 225),
    //             (95, 228),
    //             (61, 160),
    //             (142, 292),
    //             (14, 96),
    //             (22, 48),
    //             (134, 340),
    //             (42, 107),
    //             (114, 281),
    //             (89, 220),
    //             (67, 168),
    //             (107, 266),
    //             (49, 122),
    //             (119, 288),
    //             (37, 100),
    //             (124, 320),
    //             (32, 68),
    //             (142, 383),
    //             (14, 5),
    //             (56, 206),
    //             (100, 182),
    //             (78, 224),
    //             (78, 164),
    //             (83, 224),
    //             (73, 164),
    //             (107, 305),
    //             (49, 83),
    //             (134, 325),
    //             (22, 63),
    //             (150, 353),
    //             (2, 9),
    //             (4, 26),
    //         ],
    //         [
    //             (0, 95),
    //             (32, 225),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 4),
    //             (31, 316),
    //             (1, 13),
    //             (32, 322),
    //             (0, 7),
    //             (32, 324),
    //             (0, 5),
    //             (32, 328),
    //             (0, 1),
    //             (32, 328),
    //             (0, 1),
    //             (32, 329),
    //             (0, 0),
    //             (23, 252),
    //             (9, 77),
    //             (23, 296),
    //             (9, 33),
    //             (23, 317),
    //             (9, 12),
    //             (23, 323),
    //             (9, 6),
    //             (23, 327),
    //             (9, 2),
    //             (23, 329),
    //             (9, 0),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (12, 133),
    //             (20, 196),
    //             (12, 79),
    //             (20, 250),
    //             (31, 158),
    //             (1, 171),
    //             (31, 237),
    //             (1, 92),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (32, 329),
    //             (0, 0),
    //             (8, 32),
    //             (24, 297),
    //             (14, 96),
    //             (18, 233),
    //             (23, 212),
    //             (9, 117),
    //             (23, 215),
    //             (9, 114),
    //             (32, 244),
    //             (0, 85),
    //             (32, 253),
    //             (0, 76),
    //             (32, 326),
    //             (0, 3),
    //             (13, 137),
    //             (19, 192),
    //             (19, 158),
    //             (13, 171),
    //             (21, 174),
    //             (11, 155),
    //             (26, 246),
    //             (6, 83),
    //             (27, 255),
    //             (5, 74),
    //             (32, 328),
    //             (0, 0),
    //             (0, 1),
    //         ],
    //         [
    //             (72, 89),
    //             (57, 126),
    //             (0, 46),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (10, 2),
    //             (153, 235),
    //             (2, 61),
    //             (153, 244),
    //             (2, 52),
    //             (155, 245),
    //             (0, 51),
    //             (155, 258),
    //             (0, 38),
    //             (155, 277),
    //             (0, 19),
    //             (155, 291),
    //             (0, 5),
    //             (126, 250),
    //             (29, 46),
    //             (126, 262),
    //             (29, 34),
    //             (126, 267),
    //             (29, 29),
    //             (126, 287),
    //             (29, 9),
    //             (126, 296),
    //             (29, 0),
    //             (129, 296),
    //             (26, 0),
    //             (137, 258),
    //             (18, 38),
    //             (139, 258),
    //             (16, 38),
    //             (139, 277),
    //             (16, 19),
    //             (139, 286),
    //             (16, 10),
    //             (143, 286),
    //             (12, 10),
    //             (143, 296),
    //             (12, 0),
    //             (154, 296),
    //             (1, 0),
    //             (55, 173),
    //             (100, 123),
    //             (0, 0),
    //             (155, 296),
    //             (0, 0),
    //             (155, 296),
    //             (0, 0),
    //             (155, 296),
    //             (0, 0),
    //             (155, 296),
    //             (51, 71),
    //             (104, 225),
    //             (94, 136),
    //             (61, 160),
    //             (141, 200),
    //             (14, 96),
    //             (21, 40),
    //             (134, 256),
    //             (41, 81),
    //             (114, 215),
    //             (88, 163),
    //             (67, 133),
    //             (106, 209),
    //             (49, 87),
    //             (118, 224),
    //             (37, 72),
    //             (123, 247),
    //             (32, 49),
    //             (141, 294),
    //             (14, 2),
    //             (55, 173),
    //             (100, 123),
    //             (77, 186),
    //             (78, 110),
    //             (82, 186),
    //             (73, 110),
    //             (106, 239),
    //             (49, 57),
    //             (133, 252),
    //             (22, 44),
    //             (149, 261),
    //             (2, 9),
    //             (4, 26),
    //         ],
    //         [
    //             (30, 112),
    //             (39, 265),
    //             (0, 12),
    //             (4, 2),
    //             (5, 0),
    //             (0, 4),
    //             (5, 5),
    //             (82, 372),
    //             (1, 28),
    //             (83, 386),
    //             (0, 14),
    //             (83, 388),
    //             (0, 12),
    //             (83, 392),
    //             (0, 8),
    //             (83, 396),
    //             (0, 4),
    //             (83, 400),
    //             (0, 0),
    //             (69, 306),
    //             (14, 94),
    //             (69, 350),
    //             (14, 50),
    //             (69, 372),
    //             (14, 28),
    //             (69, 389),
    //             (14, 11),
    //             (69, 398),
    //             (14, 2),
    //             (70, 400),
    //             (13, 0),
    //             (72, 391),
    //             (11, 9),
    //             (74, 391),
    //             (9, 9),
    //             (74, 395),
    //             (9, 5),
    //             (74, 397),
    //             (9, 3),
    //             (78, 397),
    //             (5, 3),
    //             (78, 400),
    //             (5, 0),
    //             (82, 400),
    //             (1, 0),
    //             (27, 172),
    //             (56, 228),
    //             (12, 79),
    //             (71, 321),
    //             (31, 158),
    //             (52, 242),
    //             (31, 237),
    //             (52, 163),
    //             (32, 329),
    //             (51, 71),
    //             (83, 400),
    //             (0, 0),
    //             (83, 400),
    //             (0, 0),
    //             (83, 400),
    //             (0, 0),
    //             (16, 48),
    //             (67, 352),
    //             (27, 120),
    //             (56, 280),
    //             (54, 255),
    //             (29, 145),
    //             (58, 268),
    //             (25, 132),
    //             (68, 302),
    //             (15, 98),
    //             (71, 313),
    //             (12, 87),
    //             (81, 397),
    //             (2, 3),
    //             (28, 176),
    //             (55, 224),
    //             (36, 201),
    //             (47, 199),
    //             (39, 217),
    //             (44, 183),
    //             (59, 305),
    //             (24, 95),
    //             (67, 314),
    //             (16, 86),
    //             (83, 396),
    //             (0, 0),
    //             (0, 4),
    //         ],
    //         [
    //             (42, 72),
    //             (50, 86),
    //             (0, 39),
    //             (7, 7),
    //             (0, 5),
    //             (0, 15),
    //             (5, 1),
    //             (102, 179),
    //             (2, 46),
    //             (102, 180),
    //             (2, 45),
    //             (104, 181),
    //             (0, 44),
    //             (104, 194),
    //             (0, 31),
    //             (104, 209),
    //             (0, 16),
    //             (104, 220),
    //             (0, 5),
    //             (80, 196),
    //             (24, 29),
    //             (80, 208),
    //             (24, 17),
    //             (80, 212),
    //             (24, 13),
    //             (80, 221),
    //             (24, 4),
    //             (80, 225),
    //             (24, 0),
    //             (82, 225),
    //             (22, 0),
    //             (97, 196),
    //             (7, 29),
    //             (97, 196),
    //             (7, 29),
    //             (97, 211),
    //             (7, 14),
    //             (97, 218),
    //             (7, 7),
    //             (97, 218),
    //             (7, 7),
    //             (97, 225),
    //             (7, 0),
    //             (104, 225),
    //             (0, 0),
    //             (40, 134),
    //             (64, 91),
    //             (0, 0),
    //             (104, 225),
    //             (0, 0),
    //             (104, 225),
    //             (0, 0),
    //             (104, 225),
    //             (0, 0),
    //             (104, 225),
    //             (0, 0),
    //             (104, 225),
    //             (43, 65),
    //             (61, 160),
    //             (90, 129),
    //             (14, 96),
    //             (13, 24),
    //             (91, 201),
    //             (28, 57),
    //             (76, 168),
    //             (57, 120),
    //             (47, 105),
    //             (71, 156),
    //             (33, 69),
    //             (82, 166),
    //             (22, 59),
    //             (84, 187),
    //             (20, 38),
    //             (92, 223),
    //             (12, 2),
    //             (40, 134),
    //             (64, 91),
    //             (60, 143),
    //             (44, 82),
    //             (64, 143),
    //             (40, 82),
    //             (73, 180),
    //             (31, 45),
    //             (93, 193),
    //             (11, 32),
    //             (98, 193),
    //             (2, 9),
    //             (4, 23),
    //         ],
    //         [
    //             (50, 132),
    //             (54, 289),
    //             (0, 27),
    //             (8, 5),
    //             (5, 0),
    //             (0, 7),
    //             (9, 5),
    //             (124, 421),
    //             (2, 44),
    //             (125, 436),
    //             (1, 29),
    //             (126, 438),
    //             (0, 27),
    //             (126, 447),
    //             (0, 18),
    //             (126, 456),
    //             (0, 9),
    //             (126, 465),
    //             (0, 0),
    //             (103, 367),
    //             (23, 98),
    //             (103, 413),
    //             (23, 52),
    //             (103, 436),
    //             (23, 29),
    //             (103, 454),
    //             (23, 11),
    //             (103, 463),
    //             (23, 2),
    //             (104, 465),
    //             (22, 0),
    //             (111, 450),
    //             (15, 15),
    //             (113, 450),
    //             (13, 15),
    //             (113, 457),
    //             (13, 8),
    //             (113, 462),
    //             (13, 3),
    //             (117, 462),
    //             (9, 3),
    //             (117, 465),
    //             (9, 0),
    //             (125, 465),
    //             (1, 0),
    //             (41, 204),
    //             (85, 261),
    //             (12, 79),
    //             (114, 386),
    //             (31, 158),
    //             (95, 307),
    //             (31, 237),
    //             (95, 228),
    //             (32, 329),
    //             (94, 136),
    //             (83, 400),
    //             (43, 65),
    //             (126, 465),
    //             (0, 0),
    //             (126, 465),
    //             (0, 0),
    //             (23, 55),
    //             (103, 410),
    //             (42, 136),
    //             (84, 329),
    //             (78, 292),
    //             (48, 173),
    //             (86, 315),
    //             (40, 150),
    //             (100, 353),
    //             (26, 112),
    //             (105, 371),
    //             (21, 94),
    //             (120, 462),
    //             (6, 3),
    //             (42, 208),
    //             (84, 257),
    //             (60, 239),
    //             (66, 226),
    //             (64, 255),
    //             (62, 210),
    //             (90, 354),
    //             (36, 111),
    //             (103, 373),
    //             (23, 92),
    //             (126, 460),
    //             (0, 1),
    //             (0, 4),
    //         ],
    //         [
    //             (22, 52),
    //             (35, 62),
    //             (0, 24),
    //             (3, 4),
    //             (0, 5),
    //             (0, 12),
    //             (1, 1),
    //             (60, 130),
    //             (1, 30),
    //             (60, 130),
    //             (1, 30),
    //             (61, 131),
    //             (0, 29),
    //             (61, 139),
    //             (0, 21),
    //             (61, 149),
    //             (0, 11),
    //             (61, 155),
    //             (0, 5),
    //             (46, 135),
    //             (15, 25),
    //             (46, 145),
    //             (15, 15),
    //             (46, 148),
    //             (15, 12),
    //             (46, 156),
    //             (15, 4),
    //             (46, 160),
    //             (15, 0),
    //             (48, 160),
    //             (13, 0),
    //             (58, 137),
    //             (3, 23),
    //             (58, 137),
    //             (3, 23),
    //             (58, 149),
    //             (3, 11),
    //             (58, 153),
    //             (3, 7),
    //             (58, 153),
    //             (3, 7),
    //             (58, 160),
    //             (3, 0),
    //             (61, 160),
    //             (0, 0),
    //             (26, 102),
    //             (35, 58),
    //             (0, 0),
    //             (61, 160),
    //             (0, 0),
    //             (61, 160),
    //             (0, 0),
    //             (61, 160),
    //             (0, 0),
    //             (61, 160),
    //             (0, 0),
    //             (61, 160),
    //             (0, 0),
    //             (61, 160),
    //             (47, 64),
    //             (14, 96),
    //             (6, 17),
    //             (55, 143),
    //             (13, 41),
    //             (48, 119),
    //             (33, 83),
    //             (28, 77),
    //             (43, 109),
    //             (18, 51),
    //             (50, 115),
    //             (11, 45),
    //             (50, 129),
    //             (11, 31),
    //             (53, 158),
    //             (8, 2),
    //             (26, 102),
    //             (35, 58),
    //             (36, 105),
    //             (25, 55),
    //             (39, 105),
    //             (22, 55),
    //             (42, 131),
    //             (19, 29),
    //             (57, 134),
    //             (4, 26),
    //             (55, 129),
    //             (2, 8),
    //             (4, 23),
    //         ],
    //         [
    //             (72, 153),
    //             (75, 318),
    //             (0, 32),
    //             (11, 9),
    //             (5, 2),
    //             (0, 10),
    //             (10, 5),
    //             (171, 478),
    //             (2, 51),
    //             (172, 493),
    //             (1, 36),
    //             (173, 495),
    //             (0, 34),
    //             (173, 506),
    //             (0, 23),
    //             (173, 518),
    //             (0, 11),
    //             (173, 527),
    //             (0, 2),
    //             (148, 417),
    //             (25, 112),
    //             (148, 473),
    //             (25, 56),
    //             (148, 498),
    //             (25, 31),
    //             (148, 517),
    //             (25, 12),
    //             (148, 527),
    //             (25, 2),
    //             (151, 529),
    //             (22, 0),
    //             (155, 500),
    //             (18, 29),
    //             (157, 500),
    //             (16, 29),
    //             (157, 510),
    //             (16, 19),
    //             (157, 519),
    //             (16, 10),
    //             (161, 519),
    //             (12, 10),
    //             (161, 529),
    //             (12, 0),
    //             (172, 529),
    //             (1, 0),
    //             (54, 241),
    //             (119, 288),
    //             (12, 79),
    //             (161, 450),
    //             (31, 158),
    //             (142, 371),
    //             (31, 237),
    //             (142, 292),
    //             (32, 329),
    //             (141, 200),
    //             (83, 400),
    //             (90, 129),
    //             (126, 465),
    //             (47, 64),
    //             (173, 529),
    //             (0, 0),
    //             (27, 60),
    //             (146, 469),
    //             (50, 150),
    //             (123, 379),
    //             (101, 325),
    //             (72, 204),
    //             (116, 354),
    //             (57, 175),
    //             (136, 398),
    //             (37, 131),
    //             (141, 419),
    //             (32, 110),
    //             (159, 525),
    //             (14, 4),
    //             (55, 245),
    //             (118, 284),
    //             (83, 279),
    //             (90, 250),
    //             (90, 295),
    //             (83, 234),
    //             (119, 403),
    //             (54, 126),
    //             (147, 424),
    //             (26, 105),
    //             (173, 522),
    //             (0, 2),
    //             (0, 5),
    //         ],
    //         [
    //             (0, 31),
    //             (14, 33),
    //             (0, 19),
    //             (0, 0),
    //             (0, 3),
    //             (0, 9),
    //             (0, 1),
    //             (13, 73),
    //             (1, 23),
    //             (13, 73),
    //             (1, 23),
    //             (14, 74),
    //             (0, 22),
    //             (14, 80),
    //             (0, 16),
    //             (14, 87),
    //             (0, 9),
    //             (14, 93),
    //             (0, 3),
    //             (1, 85),
    //             (13, 11),
    //             (1, 85),
    //             (13, 11),
    //             (1, 86),
    //             (13, 10),
    //             (1, 93),
    //             (13, 3),
    //             (1, 96),
    //             (13, 0),
    //             (1, 96),
    //             (13, 0),
    //             (14, 87),
    //             (0, 9),
    //             (14, 87),
    //             (0, 9),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (13, 65),
    //             (1, 31),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (0, 0),
    //             (14, 96),
    //             (2, 12),
    //             (12, 84),
    //             (5, 27),
    //             (9, 69),
    //             (10, 50),
    //             (4, 46),
    //             (13, 70),
    //             (1, 26),
    //             (14, 70),
    //             (0, 26),
    //             (14, 81),
    //             (0, 15),
    //             (14, 95),
    //             (0, 1),
    //             (13, 65),
    //             (1, 31),
    //             (13, 65),
    //             (1, 31),
    //             (13, 65),
    //             (1, 31),
    //             (13, 82),
    //             (1, 14),
    //             (13, 83),
    //             (1, 13),
    //             (8, 67),
    //             (2, 7),
    //             (4, 22),
    //         ],
    //         [
    //             (5, 17),
    //             (19, 53),
    //             (0, 1),
    //             (0, 1),
    //             (3, 0),
    //             (0, 0),
    //             (2, 0),
    //             (29, 67),
    //             (0, 5),
    //             (29, 70),
    //             (0, 2),
    //             (29, 71),
    //             (0, 1),
    //             (29, 71),
    //             (0, 1),
    //             (29, 71),
    //             (0, 1),
    //             (29, 72),
    //             (0, 0),
    //             (24, 48),
    //             (5, 24),
    //             (24, 51),
    //             (5, 21),
    //             (24, 54),
    //             (5, 18),
    //             (24, 68),
    //             (5, 4),
    //             (24, 70),
    //             (5, 2),
    //             (24, 72),
    //             (5, 0),
    //             (26, 71),
    //             (3, 1),
    //             (26, 71),
    //             (3, 1),
    //             (26, 71),
    //             (3, 1),
    //             (26, 72),
    //             (3, 0),
    //             (29, 72),
    //             (0, 0),
    //             (29, 72),
    //             (0, 0),
    //             (29, 72),
    //             (0, 0),
    //             (17, 46),
    //             (12, 26),
    //             (1, 11),
    //             (28, 61),
    //             (7, 18),
    //             (22, 54),
    //             (7, 24),
    //             (22, 48),
    //             (8, 32),
    //             (21, 40),
    //             (16, 48),
    //             (13, 24),
    //             (23, 55),
    //             (6, 17),
    //             (27, 60),
    //             (2, 12),
    //             (29, 72),
    //             (0, 0),
    //             (29, 72),
    //             (0, 0),
    //             (29, 72),
    //             (0, 0),
    //             (29, 72),
    //             (0, 0),
    //             (29, 72),
    //             (0, 0),
    //             (29, 72),
    //             (0, 0),
    //             (29, 72),
    //             (0, 0),
    //             (17, 46),
    //             (12, 26),
    //             (23, 53),
    //             (6, 19),
    //             (23, 53),
    //             (6, 19),
    //             (25, 61),
    //             (4, 11),
    //             (28, 63),
    //             (1, 9),
    //             (29, 72),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (67, 167),
    //             (70, 298),
    //             (0, 50),
    //             (11, 8),
    //             (2, 5),
    //             (0, 19),
    //             (8, 6),
    //             (155, 484),
    //             (3, 69),
    //             (156, 496),
    //             (2, 57),
    //             (158, 498),
    //             (0, 55),
    //             (158, 515),
    //             (0, 38),
    //             (158, 534),
    //             (0, 19),
    //             (158, 548),
    //             (0, 5),
    //             (125, 454),
    //             (33, 99),
    //             (125, 507),
    //             (33, 46),
    //             (125, 530),
    //             (33, 23),
    //             (125, 542),
    //             (33, 11),
    //             (125, 553),
    //             (33, 0),
    //             (128, 553),
    //             (30, 0),
    //             (143, 516),
    //             (15, 37),
    //             (145, 516),
    //             (13, 37),
    //             (145, 535),
    //             (13, 18),
    //             (145, 543),
    //             (13, 10),
    //             (146, 543),
    //             (12, 10),
    //             (146, 553),
    //             (12, 0),
    //             (157, 553),
    //             (1, 0),
    //             (50, 260),
    //             (108, 293),
    //             (11, 68),
    //             (147, 485),
    //             (24, 140),
    //             (134, 413),
    //             (24, 213),
    //             (134, 340),
    //             (24, 297),
    //             (134, 256),
    //             (67, 352),
    //             (91, 201),
    //             (103, 410),
    //             (55, 143),
    //             (146, 469),
    //             (12, 84),
    //             (0, 0),
    //             (158, 553),
    //             (26, 105),
    //             (132, 448),
    //             (82, 303),
    //             (76, 250),
    //             (100, 352),
    //             (58, 201),
    //             (121, 396),
    //             (37, 157),
    //             (126, 428),
    //             (32, 125),
    //             (144, 548),
    //             (14, 5),
    //             (51, 264),
    //             (107, 289),
    //             (73, 291),
    //             (85, 262),
    //             (80, 307),
    //             (78, 246),
    //             (107, 424),
    //             (51, 129),
    //             (132, 444),
    //             (26, 109),
    //             (152, 517),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (10, 55),
    //             (37, 112),
    //             (0, 1),
    //             (0, 4),
    //             (5, 1),
    //             (0, 2),
    //             (3, 2),
    //             (54, 170),
    //             (1, 7),
    //             (54, 173),
    //             (1, 4),
    //             (55, 175),
    //             (0, 2),
    //             (55, 175),
    //             (0, 2),
    //             (55, 175),
    //             (0, 2),
    //             (55, 176),
    //             (0, 1),
    //             (43, 142),
    //             (12, 35),
    //             (43, 150),
    //             (12, 27),
    //             (43, 154),
    //             (12, 23),
    //             (43, 172),
    //             (12, 5),
    //             (43, 175),
    //             (12, 2),
    //             (43, 177),
    //             (12, 0),
    //             (50, 170),
    //             (5, 7),
    //             (50, 170),
    //             (5, 7),
    //             (50, 172),
    //             (5, 5),
    //             (50, 176),
    //             (5, 1),
    //             (54, 176),
    //             (1, 1),
    //             (54, 177),
    //             (1, 0),
    //             (54, 177),
    //             (1, 0),
    //             (35, 91),
    //             (20, 86),
    //             (3, 20),
    //             (52, 157),
    //             (13, 46),
    //             (42, 131),
    //             (13, 70),
    //             (42, 107),
    //             (14, 96),
    //             (41, 81),
    //             (27, 120),
    //             (28, 57),
    //             (42, 136),
    //             (13, 41),
    //             (50, 150),
    //             (5, 27),
    //             (29, 72),
    //             (26, 105),
    //             (55, 177),
    //             (0, 0),
    //             (55, 177),
    //             (0, 0),
    //             (55, 177),
    //             (0, 0),
    //             (55, 177),
    //             (0, 0),
    //             (55, 177),
    //             (0, 0),
    //             (55, 177),
    //             (0, 0),
    //             (35, 95),
    //             (20, 82),
    //             (46, 115),
    //             (9, 62),
    //             (46, 115),
    //             (9, 62),
    //             (50, 141),
    //             (5, 36),
    //             (54, 151),
    //             (1, 26),
    //             (55, 177),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (62, 129),
    //             (52, 239),
    //             (0, 50),
    //             (11, 5),
    //             (0, 4),
    //             (0, 17),
    //             (7, 4),
    //             (130, 381),
    //             (2, 67),
    //             (131, 393),
    //             (1, 55),
    //             (132, 394),
    //             (0, 54),
    //             (132, 411),
    //             (0, 37),
    //             (132, 430),
    //             (0, 18),
    //             (132, 444),
    //             (0, 4),
    //             (106, 360),
    //             (26, 88),
    //             (106, 408),
    //             (26, 40),
    //             (106, 430),
    //             (26, 18),
    //             (106, 438),
    //             (26, 10),
    //             (106, 448),
    //             (26, 0),
    //             (109, 448),
    //             (23, 0),
    //             (119, 417),
    //             (13, 31),
    //             (121, 417),
    //             (11, 31),
    //             (121, 434),
    //             (11, 14),
    //             (121, 439),
    //             (11, 9),
    //             (121, 439),
    //             (11, 9),
    //             (121, 448),
    //             (11, 0),
    //             (132, 448),
    //             (0, 0),
    //             (32, 215),
    //             (100, 233),
    //             (9, 59),
    //             (123, 389),
    //             (18, 112),
    //             (114, 336),
    //             (18, 167),
    //             (114, 281),
    //             (18, 233),
    //             (114, 215),
    //             (56, 280),
    //             (76, 168),
    //             (84, 329),
    //             (48, 119),
    //             (123, 379),
    //             (9, 69),
    //             (0, 0),
    //             (132, 448),
    //             (0, 0),
    //             (132, 448),
    //             (56, 198),
    //             (76, 250),
    //             (74, 247),
    //             (58, 201),
    //             (95, 291),
    //             (37, 157),
    //             (100, 323),
    //             (32, 125),
    //             (118, 443),
    //             (14, 5),
    //             (33, 215),
    //             (99, 233),
    //             (50, 229),
    //             (82, 219),
    //             (57, 245),
    //             (75, 203),
    //             (82, 344),
    //             (50, 104),
    //             (106, 356),
    //             (26, 92),
    //             (126, 412),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (37, 115),
    //             (62, 228),
    //             (0, 11),
    //             (2, 6),
    //             (5, 3),
    //             (0, 9),
    //             (5, 3),
    //             (109, 349),
    //             (2, 26),
    //             (110, 358),
    //             (1, 17),
    //             (111, 361),
    //             (0, 14),
    //             (111, 365),
    //             (0, 10),
    //             (111, 367),
    //             (0, 8),
    //             (111, 372),
    //             (0, 3),
    //             (89, 297),
    //             (22, 78),
    //             (89, 329),
    //             (22, 46),
    //             (89, 343),
    //             (22, 32),
    //             (89, 364),
    //             (22, 11),
    //             (89, 373),
    //             (22, 2),
    //             (90, 375),
    //             (21, 0),
    //             (104, 356),
    //             (7, 19),
    //             (104, 356),
    //             (7, 19),
    //             (104, 365),
    //             (7, 10),
    //             (104, 371),
    //             (7, 4),
    //             (108, 371),
    //             (3, 4),
    //             (108, 375),
    //             (3, 0),
    //             (110, 375),
    //             (1, 0),
    //             (49, 182),
    //             (62, 193),
    //             (6, 50),
    //             (105, 325),
    //             (22, 107),
    //             (89, 268),
    //             (22, 155),
    //             (89, 220),
    //             (23, 212),
    //             (88, 163),
    //             (54, 255),
    //             (57, 120),
    //             (78, 292),
    //             (33, 83),
    //             (101, 325),
    //             (10, 50),
    //             (29, 72),
    //             (82, 303),
    //             (55, 177),
    //             (56, 198),
    //             (111, 375),
    //             (0, 0),
    //             (111, 375),
    //             (0, 0),
    //             (111, 375),
    //             (0, 0),
    //             (111, 375),
    //             (0, 0),
    //             (111, 375),
    //             (0, 0),
    //             (49, 186),
    //             (62, 189),
    //             (67, 213),
    //             (44, 162),
    //             (70, 219),
    //             (41, 156),
    //             (86, 288),
    //             (25, 87),
    //             (97, 306),
    //             (14, 69),
    //             (108, 364),
    //             (2, 1),
    //             (1, 10),
    //         ],
    //         [
    //             (35, 69),
    //             (27, 123),
    //             (0, 40),
    //             (9, 3),
    //             (0, 2),
    //             (0, 10),
    //             (5, 3),
    //             (75, 202),
    //             (1, 48),
    //             (75, 208),
    //             (1, 42),
    //             (76, 208),
    //             (0, 42),
    //             (76, 221),
    //             (0, 29),
    //             (76, 238),
    //             (0, 12),
    //             (76, 248),
    //             (0, 2),
    //             (60, 205),
    //             (16, 45),
    //             (60, 229),
    //             (16, 21),
    //             (60, 241),
    //             (16, 9),
    //             (60, 246),
    //             (16, 4),
    //             (60, 250),
    //             (16, 0),
    //             (62, 250),
    //             (14, 0),
    //             (65, 231),
    //             (11, 19),
    //             (67, 231),
    //             (9, 19),
    //             (67, 241),
    //             (9, 9),
    //             (67, 244),
    //             (9, 6),
    //             (67, 244),
    //             (9, 6),
    //             (67, 250),
    //             (9, 0),
    //             (76, 250),
    //             (0, 0),
    //             (18, 124),
    //             (58, 126),
    //             (6, 29),
    //             (70, 221),
    //             (9, 51),
    //             (67, 199),
    //             (9, 82),
    //             (67, 168),
    //             (9, 117),
    //             (67, 133),
    //             (29, 145),
    //             (47, 105),
    //             (48, 173),
    //             (28, 77),
    //             (72, 204),
    //             (4, 46),
    //             (0, 0),
    //             (76, 250),
    //             (0, 0),
    //             (76, 250),
    //             (0, 0),
    //             (76, 250),
    //             (18, 49),
    //             (58, 201),
    //             (39, 93),
    //             (37, 157),
    //             (44, 125),
    //             (32, 125),
    //             (62, 245),
    //             (14, 5),
    //             (19, 124),
    //             (57, 126),
    //             (29, 131),
    //             (47, 119),
    //             (33, 141),
    //             (43, 109),
    //             (46, 197),
    //             (30, 53),
    //             (63, 201),
    //             (13, 49),
    //             (73, 225),
    //             (0, 8),
    //             (3, 17),
    //         ],
    //         [
    //             (44, 121),
    //             (65, 230),
    //             (0, 51),
    //             (10, 6),
    //             (5, 4),
    //             (0, 9),
    //             (5, 3),
    //             (127, 357),
    //             (2, 67),
    //             (128, 366),
    //             (1, 58),
    //             (129, 369),
    //             (0, 55),
    //             (129, 386),
    //             (0, 38),
    //             (129, 405),
    //             (0, 19),
    //             (129, 420),
    //             (0, 4),
    //             (104, 345),
    //             (25, 79),
    //             (104, 377),
    //             (25, 47),
    //             (104, 391),
    //             (25, 33),
    //             (104, 413),
    //             (25, 11),
    //             (104, 422),
    //             (25, 2),
    //             (105, 424),
    //             (24, 0),
    //             (112, 405),
    //             (17, 19),
    //             (114, 405),
    //             (15, 19),
    //             (114, 414),
    //             (15, 10),
    //             (114, 420),
    //             (15, 4),
    //             (118, 420),
    //             (11, 4),
    //             (118, 424),
    //             (11, 0),
    //             (128, 424),
    //             (1, 0),
    //             (58, 230),
    //             (71, 194),
    //             (6, 50),
    //             (123, 374),
    //             (22, 110),
    //             (107, 314),
    //             (22, 158),
    //             (107, 266),
    //             (23, 215),
    //             (106, 209),
    //             (58, 268),
    //             (71, 156),
    //             (86, 315),
    //             (43, 109),
    //             (116, 354),
    //             (13, 70),
    //             (29, 72),
    //             (100, 352),
    //             (55, 177),
    //             (74, 247),
    //             (111, 375),
    //             (18, 49),
    //             (129, 424),
    //             (0, 0),
    //             (129, 424),
    //             (0, 0),
    //             (129, 424),
    //             (0, 0),
    //             (129, 424),
    //             (0, 0),
    //             (58, 234),
    //             (71, 190),
    //             (76, 261),
    //             (53, 163),
    //             (79, 267),
    //             (50, 157),
    //             (95, 336),
    //             (34, 88),
    //             (115, 354),
    //             (14, 70),
    //             (123, 395),
    //             (2, 7),
    //             (4, 22),
    //         ],
    //         [
    //             (28, 63),
    //             (24, 121),
    //             (0, 0),
    //             (1, 3),
    //             (0, 1),
    //             (0, 10),
    //             (5, 3),
    //             (57, 194),
    //             (1, 7),
    //             (57, 200),
    //             (1, 1),
    //             (58, 200),
    //             (0, 1),
    //             (58, 200),
    //             (0, 1),
    //             (58, 200),
    //             (0, 1),
    //             (58, 200),
    //             (0, 1),
    //             (45, 157),
    //             (13, 44),
    //             (45, 181),
    //             (13, 20),
    //             (45, 193),
    //             (13, 8),
    //             (45, 197),
    //             (13, 4),
    //             (45, 201),
    //             (13, 0),
    //             (47, 201),
    //             (11, 0),
    //             (57, 182),
    //             (1, 19),
    //             (57, 182),
    //             (1, 19),
    //             (57, 192),
    //             (1, 9),
    //             (57, 195),
    //             (1, 6),
    //             (57, 195),
    //             (1, 6),
    //             (57, 201),
    //             (1, 0),
    //             (58, 201),
    //             (0, 0),
    //             (9, 76),
    //             (49, 125),
    //             (6, 29),
    //             (52, 172),
    //             (9, 48),
    //             (49, 153),
    //             (9, 79),
    //             (49, 122),
    //             (9, 114),
    //             (49, 87),
    //             (25, 132),
    //             (33, 69),
    //             (40, 150),
    //             (18, 51),
    //             (57, 175),
    //             (1, 26),
    //             (0, 0),
    //             (58, 201),
    //             (0, 0),
    //             (58, 201),
    //             (0, 0),
    //             (58, 201),
    //             (0, 0),
    //             (58, 201),
    //             (21, 44),
    //             (37, 157),
    //             (26, 76),
    //             (32, 125),
    //             (44, 196),
    //             (14, 5),
    //             (10, 76),
    //             (48, 125),
    //             (20, 83),
    //             (38, 118),
    //             (24, 93),
    //             (34, 108),
    //             (37, 149),
    //             (21, 52),
    //             (45, 153),
    //             (13, 48),
    //             (58, 194),
    //             (0, 2),
    //             (0, 5),
    //         ],
    //         [
    //             (44, 130),
    //             (86, 265),
    //             (0, 51),
    //             (10, 6),
    //             (5, 4),
    //             (0, 9),
    //             (5, 3),
    //             (148, 398),
    //             (2, 70),
    //             (149, 410),
    //             (1, 58),
    //             (150, 413),
    //             (0, 55),
    //             (150, 430),
    //             (0, 38),
    //             (150, 449),
    //             (0, 19),
    //             (150, 464),
    //             (0, 4),
    //             (114, 373),
    //             (36, 95),
    //             (114, 408),
    //             (36, 60),
    //             (114, 432),
    //             (36, 36),
    //             (114, 455),
    //             (36, 13),
    //             (114, 466),
    //             (36, 2),
    //             (115, 468),
    //             (35, 0),
    //             (133, 448),
    //             (17, 20),
    //             (135, 448),
    //             (15, 20),
    //             (135, 457),
    //             (15, 11),
    //             (135, 463),
    //             (15, 5),
    //             (139, 463),
    //             (11, 5),
    //             (139, 468),
    //             (11, 0),
    //             (149, 468),
    //             (1, 0),
    //             (63, 247),
    //             (87, 221),
    //             (12, 58),
    //             (138, 410),
    //             (31, 128),
    //             (119, 340),
    //             (31, 180),
    //             (119, 288),
    //             (32, 244),
    //             (118, 224),
    //             (68, 302),
    //             (82, 166),
    //             (100, 353),
    //             (50, 115),
    //             (136, 398),
    //             (14, 70),
    //             (29, 72),
    //             (121, 396),
    //             (55, 177),
    //             (95, 291),
    //             (111, 375),
    //             (39, 93),
    //             (129, 424),
    //             (21, 44),
    //             (150, 468),
    //             (0, 0),
    //             (150, 468),
    //             (0, 0),
    //             (150, 468),
    //             (0, 0),
    //             (64, 251),
    //             (86, 217),
    //             (85, 279),
    //             (65, 189),
    //             (91, 288),
    //             (59, 180),
    //             (112, 368),
    //             (38, 100),
    //             (132, 387),
    //             (18, 81),
    //             (144, 438),
    //             (2, 7),
    //             (4, 23),
    //         ],
    //         [
    //             (28, 54),
    //             (3, 86),
    //             (0, 0),
    //             (1, 3),
    //             (0, 1),
    //             (0, 10),
    //             (5, 3),
    //             (36, 153),
    //             (1, 4),
    //             (36, 156),
    //             (1, 1),
    //             (37, 156),
    //             (0, 1),
    //             (37, 156),
    //             (0, 1),
    //             (37, 156),
    //             (0, 1),
    //             (37, 156),
    //             (0, 1),
    //             (35, 129),
    //             (2, 28),
    //             (35, 150),
    //             (2, 7),
    //             (35, 152),
    //             (2, 5),
    //             (35, 155),
    //             (2, 2),
    //             (35, 157),
    //             (2, 0),
    //             (37, 157),
    //             (0, 0),
    //             (36, 139),
    //             (1, 18),
    //             (36, 139),
    //             (1, 18),
    //             (36, 149),
    //             (1, 8),
    //             (36, 152),
    //             (1, 5),
    //             (36, 152),
    //             (1, 5),
    //             (36, 157),
    //             (1, 0),
    //             (37, 157),
    //             (0, 0),
    //             (4, 59),
    //             (33, 98),
    //             (0, 21),
    //             (37, 136),
    //             (0, 30),
    //             (37, 127),
    //             (0, 57),
    //             (37, 100),
    //             (0, 85),
    //             (37, 72),
    //             (15, 98),
    //             (22, 59),
    //             (26, 112),
    //             (11, 45),
    //             (37, 131),
    //             (0, 26),
    //             (0, 0),
    //             (37, 157),
    //             (0, 0),
    //             (37, 157),
    //             (0, 0),
    //             (37, 157),
    //             (0, 0),
    //             (37, 157),
    //             (0, 0),
    //             (37, 157),
    //             (5, 32),
    //             (32, 125),
    //             (23, 152),
    //             (14, 5),
    //             (4, 59),
    //             (33, 98),
    //             (11, 65),
    //             (26, 92),
    //             (12, 72),
    //             (25, 85),
    //             (20, 117),
    //             (17, 40),
    //             (28, 120),
    //             (9, 37),
    //             (37, 151),
    //             (0, 2),
    //             (0, 4),
    //         ],
    //         [
    //             (49, 142),
    //             (86, 274),
    //             (0, 51),
    //             (10, 7),
    //             (5, 4),
    //             (0, 19),
    //             (5, 3),
    //             (153, 430),
    //             (2, 70),
    //             (154, 442),
    //             (1, 58),
    //             (155, 445),
    //             (0, 55),
    //             (155, 462),
    //             (0, 38),
    //             (155, 481),
    //             (0, 19),
    //             (155, 496),
    //             (0, 4),
    //             (119, 403),
    //             (36, 97),
    //             (119, 440),
    //             (36, 60),
    //             (119, 464),
    //             (36, 36),
    //             (119, 487),
    //             (36, 13),
    //             (119, 498),
    //             (36, 2),
    //             (120, 500),
    //             (35, 0),
    //             (138, 469),
    //             (17, 31),
    //             (140, 469),
    //             (15, 31),
    //             (140, 488),
    //             (15, 12),
    //             (140, 495),
    //             (15, 5),
    //             (144, 495),
    //             (11, 5),
    //             (144, 500),
    //             (11, 0),
    //             (154, 500),
    //             (1, 0),
    //             (63, 253),
    //             (92, 247),
    //             (12, 58),
    //             (143, 442),
    //             (31, 128),
    //             (124, 372),
    //             (31, 180),
    //             (124, 320),
    //             (32, 253),
    //             (123, 247),
    //             (71, 313),
    //             (84, 187),
    //             (105, 371),
    //             (50, 129),
    //             (141, 419),
    //             (14, 81),
    //             (29, 72),
    //             (126, 428),
    //             (55, 177),
    //             (100, 323),
    //             (111, 375),
    //             (44, 125),
    //             (129, 424),
    //             (26, 76),
    //             (150, 468),
    //             (5, 32),
    //             (155, 500),
    //             (0, 0),
    //             (155, 500),
    //             (0, 0),
    //             (64, 257),
    //             (91, 243),
    //             (85, 285),
    //             (70, 215),
    //             (91, 294),
    //             (64, 206),
    //             (115, 388),
    //             (40, 112),
    //             (135, 408),
    //             (20, 92),
    //             (149, 468),
    //             (2, 8),
    //             (4, 24),
    //         ],
    //         [
    //             (23, 42),
    //             (3, 77),
    //             (0, 0),
    //             (1, 2),
    //             (0, 1),
    //             (0, 0),
    //             (5, 3),
    //             (31, 121),
    //             (1, 4),
    //             (31, 124),
    //             (1, 1),
    //             (32, 124),
    //             (0, 1),
    //             (32, 124),
    //             (0, 1),
    //             (32, 124),
    //             (0, 1),
    //             (32, 124),
    //             (0, 1),
    //             (30, 99),
    //             (2, 26),
    //             (30, 118),
    //             (2, 7),
    //             (30, 120),
    //             (2, 5),
    //             (30, 123),
    //             (2, 2),
    //             (30, 125),
    //             (2, 0),
    //             (32, 125),
    //             (0, 0),
    //             (31, 118),
    //             (1, 7),
    //             (31, 118),
    //             (1, 7),
    //             (31, 118),
    //             (1, 7),
    //             (31, 120),
    //             (1, 5),
    //             (31, 120),
    //             (1, 5),
    //             (31, 125),
    //             (1, 0),
    //             (32, 125),
    //             (0, 0),
    //             (4, 53),
    //             (28, 72),
    //             (0, 21),
    //             (32, 104),
    //             (0, 30),
    //             (32, 95),
    //             (0, 57),
    //             (32, 68),
    //             (0, 76),
    //             (32, 49),
    //             (12, 87),
    //             (20, 38),
    //             (21, 94),
    //             (11, 31),
    //             (32, 110),
    //             (0, 15),
    //             (0, 0),
    //             (32, 125),
    //             (0, 0),
    //             (32, 125),
    //             (0, 0),
    //             (32, 125),
    //             (0, 0),
    //             (32, 125),
    //             (0, 0),
    //             (32, 125),
    //             (0, 0),
    //             (32, 125),
    //             (18, 120),
    //             (14, 5),
    //             (4, 53),
    //             (28, 72),
    //             (11, 59),
    //             (21, 66),
    //             (12, 66),
    //             (20, 59),
    //             (17, 97),
    //             (15, 28),
    //             (25, 99),
    //             (7, 26),
    //             (32, 121),
    //             (0, 1),
    //             (0, 3),
    //         ],
    //         [
    //             (60, 184),
    //             (89, 346),
    //             (0, 51),
    //             (11, 9),
    //             (5, 5),
    //             (0, 19),
    //             (8, 6),
    //             (170, 546),
    //             (3, 74),
    //             (171, 561),
    //             (2, 59),
    //             (173, 564),
    //             (0, 56),
    //             (173, 581),
    //             (0, 39),
    //             (173, 600),
    //             (0, 20),
    //             (173, 615),
    //             (0, 5),
    //             (135, 497),
    //             (38, 123),
    //             (135, 553),
    //             (38, 67),
    //             (135, 579),
    //             (38, 41),
    //             (135, 605),
    //             (38, 15),
    //             (135, 618),
    //             (38, 2),
    //             (138, 620),
    //             (35, 0),
    //             (155, 583),
    //             (18, 37),
    //             (157, 583),
    //             (16, 37),
    //             (157, 602),
    //             (16, 18),
    //             (157, 611),
    //             (16, 9),
    //             (161, 611),
    //             (12, 9),
    //             (161, 620),
    //             (12, 0),
    //             (172, 620),
    //             (1, 0),
    //             (65, 305),
    //             (108, 315),
    //             (12, 79),
    //             (161, 541),
    //             (31, 158),
    //             (142, 462),
    //             (31, 237),
    //             (142, 383),
    //             (32, 326),
    //             (141, 294),
    //             (81, 397),
    //             (92, 223),
    //             (120, 462),
    //             (53, 158),
    //             (159, 525),
    //             (14, 95),
    //             (29, 72),
    //             (144, 548),
    //             (55, 177),
    //             (118, 443),
    //             (111, 375),
    //             (62, 245),
    //             (129, 424),
    //             (44, 196),
    //             (150, 468),
    //             (23, 152),
    //             (155, 500),
    //             (18, 120),
    //             (173, 620),
    //             (0, 0),
    //             (66, 309),
    //             (107, 311),
    //             (89, 343),
    //             (84, 277),
    //             (96, 359),
    //             (77, 261),
    //             (125, 481),
    //             (48, 139),
    //             (146, 503),
    //             (27, 117),
    //             (167, 584),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (12, 0),
    //             (0, 5),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (2, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 4),
    //             (0, 1),
    //             (14, 4),
    //             (0, 1),
    //             (14, 4),
    //             (0, 1),
    //             (14, 4),
    //             (0, 1),
    //             (14, 4),
    //             (0, 1),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (2, 1),
    //             (12, 4),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 3),
    //             (14, 2),
    //             (2, 3),
    //             (12, 2),
    //             (6, 3),
    //             (8, 2),
    //             (14, 4),
    //             (0, 1),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (0, 0),
    //             (14, 5),
    //             (2, 1),
    //             (12, 4),
    //             (7, 1),
    //             (7, 4),
    //             (7, 1),
    //             (7, 4),
    //             (7, 4),
    //             (7, 1),
    //             (14, 4),
    //             (0, 1),
    //             (14, 5),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (12, 70),
    //             (44, 169),
    //             (0, 51),
    //             (4, 9),
    //             (5, 1),
    //             (0, 7),
    //             (3, 3),
    //             (65, 247),
    //             (3, 63),
    //             (66, 255),
    //             (2, 55),
    //             (68, 258),
    //             (0, 52),
    //             (68, 275),
    //             (0, 35),
    //             (68, 294),
    //             (0, 16),
    //             (68, 309),
    //             (0, 1),
    //             (46, 240),
    //             (22, 70),
    //             (46, 259),
    //             (22, 51),
    //             (46, 277),
    //             (22, 33),
    //             (46, 295),
    //             (22, 15),
    //             (46, 308),
    //             (22, 2),
    //             (46, 310),
    //             (22, 0),
    //             (57, 286),
    //             (11, 24),
    //             (59, 286),
    //             (9, 24),
    //             (59, 293),
    //             (9, 17),
    //             (59, 302),
    //             (9, 8),
    //             (63, 302),
    //             (5, 8),
    //             (63, 310),
    //             (5, 0),
    //             (67, 310),
    //             (1, 0),
    //             (67, 306),
    //             (1, 4),
    //             (5, 38),
    //             (63, 272),
    //             (12, 71),
    //             (56, 239),
    //             (12, 104),
    //             (56, 206),
    //             (13, 137),
    //             (55, 173),
    //             (28, 176),
    //             (40, 134),
    //             (42, 208),
    //             (26, 102),
    //             (55, 245),
    //             (13, 65),
    //             (17, 46),
    //             (51, 264),
    //             (35, 95),
    //             (33, 215),
    //             (49, 186),
    //             (19, 124),
    //             (58, 234),
    //             (10, 76),
    //             (64, 251),
    //             (4, 59),
    //             (64, 257),
    //             (4, 53),
    //             (66, 309),
    //             (2, 1),
    //             (68, 310),
    //             (0, 0),
    //             (68, 310),
    //             (0, 0),
    //             (68, 310),
    //             (0, 0),
    //             (68, 310),
    //             (0, 0),
    //             (68, 310),
    //             (0, 0),
    //             (62, 274),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (60, 114),
    //             (45, 182),
    //             (0, 0),
    //             (7, 0),
    //             (0, 4),
    //             (0, 12),
    //             (7, 3),
    //             (119, 304),
    //             (0, 11),
    //             (119, 311),
    //             (0, 4),
    //             (119, 311),
    //             (0, 4),
    //             (119, 311),
    //             (0, 4),
    //             (119, 311),
    //             (0, 4),
    //             (119, 311),
    //             (0, 4),
    //             (103, 262),
    //             (16, 53),
    //             (103, 299),
    //             (16, 16),
    //             (103, 307),
    //             (16, 8),
    //             (103, 315),
    //             (16, 0),
    //             (103, 315),
    //             (16, 0),
    //             (106, 315),
    //             (13, 0),
    //             (112, 301),
    //             (7, 14),
    //             (112, 301),
    //             (7, 14),
    //             (112, 313),
    //             (7, 2),
    //             (112, 313),
    //             (7, 2),
    //             (112, 313),
    //             (7, 2),
    //             (112, 315),
    //             (7, 0),
    //             (119, 315),
    //             (0, 0),
    //             (0, 0),
    //             (119, 315),
    //             (7, 41),
    //             (112, 274),
    //             (19, 87),
    //             (100, 228),
    //             (19, 133),
    //             (100, 182),
    //             (19, 192),
    //             (100, 123),
    //             (55, 224),
    //             (64, 91),
    //             (84, 257),
    //             (35, 58),
    //             (118, 284),
    //             (1, 31),
    //             (12, 26),
    //             (107, 289),
    //             (20, 82),
    //             (99, 233),
    //             (62, 189),
    //             (57, 126),
    //             (71, 190),
    //             (48, 125),
    //             (86, 217),
    //             (33, 98),
    //             (91, 243),
    //             (28, 72),
    //             (107, 311),
    //             (12, 4),
    //             (0, 0),
    //             (119, 315),
    //             (28, 34),
    //             (91, 281),
    //             (35, 50),
    //             (84, 265),
    //             (64, 175),
    //             (55, 140),
    //             (92, 197),
    //             (27, 118),
    //             (119, 315),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (23, 86),
    //             (59, 187),
    //             (0, 51),
    //             (4, 9),
    //             (5, 1),
    //             (0, 7),
    //             (5, 3),
    //             (93, 281),
    //             (3, 63),
    //             (94, 289),
    //             (2, 55),
    //             (96, 292),
    //             (0, 52),
    //             (96, 309),
    //             (0, 35),
    //             (96, 328),
    //             (0, 16),
    //             (96, 343),
    //             (0, 1),
    //             (69, 265),
    //             (27, 79),
    //             (69, 289),
    //             (27, 55),
    //             (69, 307),
    //             (27, 37),
    //             (69, 329),
    //             (27, 15),
    //             (69, 342),
    //             (27, 2),
    //             (70, 344),
    //             (26, 0),
    //             (85, 320),
    //             (11, 24),
    //             (87, 320),
    //             (9, 24),
    //             (87, 327),
    //             (9, 17),
    //             (87, 336),
    //             (9, 8),
    //             (91, 336),
    //             (5, 8),
    //             (91, 344),
    //             (5, 0),
    //             (95, 344),
    //             (1, 0),
    //             (67, 306),
    //             (29, 38),
    //             (6, 43),
    //             (90, 301),
    //             (18, 81),
    //             (78, 263),
    //             (18, 120),
    //             (78, 224),
    //             (19, 158),
    //             (77, 186),
    //             (36, 201),
    //             (60, 143),
    //             (60, 239),
    //             (36, 105),
    //             (83, 279),
    //             (13, 65),
    //             (23, 53),
    //             (73, 291),
    //             (46, 115),
    //             (50, 229),
    //             (67, 213),
    //             (29, 131),
    //             (76, 261),
    //             (20, 83),
    //             (85, 279),
    //             (11, 65),
    //             (85, 285),
    //             (11, 59),
    //             (89, 343),
    //             (7, 1),
    //             (68, 310),
    //             (28, 34),
    //             (96, 344),
    //             (0, 0),
    //             (96, 344),
    //             (0, 0),
    //             (96, 344),
    //             (0, 0),
    //             (96, 344),
    //             (0, 0),
    //             (90, 308),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (49, 98),
    //             (30, 164),
    //             (0, 0),
    //             (7, 0),
    //             (0, 4),
    //             (0, 12),
    //             (5, 3),
    //             (91, 270),
    //             (0, 11),
    //             (91, 277),
    //             (0, 4),
    //             (91, 277),
    //             (0, 4),
    //             (91, 277),
    //             (0, 4),
    //             (91, 277),
    //             (0, 4),
    //             (91, 277),
    //             (0, 4),
    //             (80, 237),
    //             (11, 44),
    //             (80, 269),
    //             (11, 12),
    //             (80, 277),
    //             (11, 4),
    //             (80, 281),
    //             (11, 0),
    //             (80, 281),
    //             (11, 0),
    //             (82, 281),
    //             (9, 0),
    //             (84, 267),
    //             (7, 14),
    //             (84, 267),
    //             (7, 14),
    //             (84, 279),
    //             (7, 2),
    //             (84, 279),
    //             (7, 2),
    //             (84, 279),
    //             (7, 2),
    //             (84, 281),
    //             (7, 0),
    //             (91, 281),
    //             (0, 0),
    //             (0, 0),
    //             (91, 281),
    //             (6, 36),
    //             (85, 245),
    //             (13, 77),
    //             (78, 204),
    //             (13, 117),
    //             (78, 164),
    //             (13, 171),
    //             (78, 110),
    //             (47, 199),
    //             (44, 82),
    //             (66, 226),
    //             (25, 55),
    //             (90, 250),
    //             (1, 31),
    //             (6, 19),
    //             (85, 262),
    //             (9, 62),
    //             (82, 219),
    //             (44, 162),
    //             (47, 119),
    //             (53, 163),
    //             (38, 118),
    //             (65, 189),
    //             (26, 92),
    //             (70, 215),
    //             (21, 66),
    //             (84, 277),
    //             (7, 4),
    //             (0, 0),
    //             (91, 281),
    //             (0, 0),
    //             (91, 281),
    //             (7, 16),
    //             (84, 265),
    //             (36, 141),
    //             (55, 140),
    //             (64, 163),
    //             (27, 118),
    //             (91, 281),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (25, 90),
    //             (64, 199),
    //             (0, 51),
    //             (4, 9),
    //             (5, 1),
    //             (0, 7),
    //             (5, 3),
    //             (100, 297),
    //             (3, 63),
    //             (101, 305),
    //             (2, 55),
    //             (103, 308),
    //             (0, 52),
    //             (103, 325),
    //             (0, 35),
    //             (103, 344),
    //             (0, 16),
    //             (103, 359),
    //             (0, 1),
    //             (76, 274),
    //             (27, 86),
    //             (76, 305),
    //             (27, 55),
    //             (76, 323),
    //             (27, 37),
    //             (76, 345),
    //             (27, 15),
    //             (76, 358),
    //             (27, 2),
    //             (77, 360),
    //             (26, 0),
    //             (92, 336),
    //             (11, 24),
    //             (94, 336),
    //             (9, 24),
    //             (94, 343),
    //             (9, 17),
    //             (94, 352),
    //             (9, 8),
    //             (98, 352),
    //             (5, 8),
    //             (98, 360),
    //             (5, 0),
    //             (102, 360),
    //             (1, 0),
    //             (67, 306),
    //             (36, 54),
    //             (6, 48),
    //             (97, 312),
    //             (20, 89),
    //             (83, 271),
    //             (20, 136),
    //             (83, 224),
    //             (21, 174),
    //             (82, 186),
    //             (39, 217),
    //             (64, 143),
    //             (64, 255),
    //             (39, 105),
    //             (90, 295),
    //             (13, 65),
    //             (23, 53),
    //             (80, 307),
    //             (46, 115),
    //             (57, 245),
    //             (70, 219),
    //             (33, 141),
    //             (79, 267),
    //             (24, 93),
    //             (91, 288),
    //             (12, 72),
    //             (91, 294),
    //             (12, 66),
    //             (96, 359),
    //             (7, 1),
    //             (68, 310),
    //             (35, 50),
    //             (96, 344),
    //             (7, 16),
    //             (103, 360),
    //             (0, 0),
    //             (103, 360),
    //             (0, 0),
    //             (103, 360),
    //             (0, 0),
    //             (97, 324),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (47, 94),
    //             (25, 152),
    //             (0, 0),
    //             (7, 0),
    //             (0, 4),
    //             (0, 12),
    //             (5, 3),
    //             (84, 254),
    //             (0, 11),
    //             (84, 261),
    //             (0, 4),
    //             (84, 261),
    //             (0, 4),
    //             (84, 261),
    //             (0, 4),
    //             (84, 261),
    //             (0, 4),
    //             (84, 261),
    //             (0, 4),
    //             (73, 228),
    //             (11, 37),
    //             (73, 253),
    //             (11, 12),
    //             (73, 261),
    //             (11, 4),
    //             (73, 265),
    //             (11, 0),
    //             (73, 265),
    //             (11, 0),
    //             (75, 265),
    //             (9, 0),
    //             (77, 251),
    //             (7, 14),
    //             (77, 251),
    //             (7, 14),
    //             (77, 263),
    //             (7, 2),
    //             (77, 263),
    //             (7, 2),
    //             (77, 263),
    //             (7, 2),
    //             (77, 265),
    //             (7, 0),
    //             (84, 265),
    //             (0, 0),
    //             (0, 0),
    //             (84, 265),
    //             (6, 31),
    //             (78, 234),
    //             (11, 69),
    //             (73, 196),
    //             (11, 101),
    //             (73, 164),
    //             (11, 155),
    //             (73, 110),
    //             (44, 183),
    //             (40, 82),
    //             (62, 210),
    //             (22, 55),
    //             (83, 234),
    //             (1, 31),
    //             (6, 19),
    //             (78, 246),
    //             (9, 62),
    //             (75, 203),
    //             (41, 156),
    //             (43, 109),
    //             (50, 157),
    //             (34, 108),
    //             (59, 180),
    //             (25, 85),
    //             (64, 206),
    //             (20, 59),
    //             (77, 261),
    //             (7, 4),
    //             (0, 0),
    //             (84, 265),
    //             (0, 0),
    //             (84, 265),
    //             (0, 0),
    //             (84, 265),
    //             (29, 125),
    //             (55, 140),
    //             (57, 147),
    //             (27, 118),
    //             (84, 265),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (41, 135),
    //             (74, 267),
    //             (0, 51),
    //             (4, 9),
    //             (5, 3),
    //             (0, 15),
    //             (8, 5),
    //             (129, 416),
    //             (3, 69),
    //             (130, 428),
    //             (2, 57),
    //             (132, 431),
    //             (0, 54),
    //             (132, 448),
    //             (0, 37),
    //             (132, 467),
    //             (0, 18),
    //             (132, 482),
    //             (0, 3),
    //             (100, 384),
    //             (32, 101),
    //             (100, 424),
    //             (32, 61),
    //             (100, 446),
    //             (32, 39),
    //             (100, 470),
    //             (32, 15),
    //             (100, 483),
    //             (32, 2),
    //             (101, 485),
    //             (31, 0),
    //             (121, 452),
    //             (11, 33),
    //             (123, 452),
    //             (9, 33),
    //             (123, 467),
    //             (9, 18),
    //             (123, 476),
    //             (9, 9),
    //             (127, 476),
    //             (5, 9),
    //             (127, 485),
    //             (5, 0),
    //             (131, 485),
    //             (1, 0),
    //             (67, 306),
    //             (65, 179),
    //             (10, 64),
    //             (122, 421),
    //             (25, 120),
    //             (107, 365),
    //             (25, 180),
    //             (107, 305),
    //             (26, 246),
    //             (106, 239),
    //             (59, 305),
    //             (73, 180),
    //             (90, 354),
    //             (42, 131),
    //             (119, 403),
    //             (13, 82),
    //             (25, 61),
    //             (107, 424),
    //             (50, 141),
    //             (82, 344),
    //             (86, 288),
    //             (46, 197),
    //             (95, 336),
    //             (37, 149),
    //             (112, 368),
    //             (20, 117),
    //             (115, 388),
    //             (17, 97),
    //             (125, 481),
    //             (7, 4),
    //             (68, 310),
    //             (64, 175),
    //             (96, 344),
    //             (36, 141),
    //             (103, 360),
    //             (29, 125),
    //             (132, 485),
    //             (0, 0),
    //             (132, 485),
    //             (0, 0),
    //             (126, 449),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (31, 49),
    //             (15, 84),
    //             (0, 0),
    //             (7, 0),
    //             (0, 2),
    //             (0, 4),
    //             (2, 1),
    //             (55, 135),
    //             (0, 5),
    //             (55, 138),
    //             (0, 2),
    //             (55, 138),
    //             (0, 2),
    //             (55, 138),
    //             (0, 2),
    //             (55, 138),
    //             (0, 2),
    //             (55, 138),
    //             (0, 2),
    //             (49, 118),
    //             (6, 22),
    //             (49, 134),
    //             (6, 6),
    //             (49, 138),
    //             (6, 2),
    //             (49, 140),
    //             (6, 0),
    //             (49, 140),
    //             (6, 0),
    //             (51, 140),
    //             (4, 0),
    //             (48, 135),
    //             (7, 5),
    //             (48, 135),
    //             (7, 5),
    //             (48, 139),
    //             (7, 1),
    //             (48, 139),
    //             (7, 1),
    //             (48, 139),
    //             (7, 1),
    //             (48, 140),
    //             (7, 0),
    //             (55, 140),
    //             (0, 0),
    //             (0, 0),
    //             (55, 140),
    //             (2, 15),
    //             (53, 125),
    //             (6, 38),
    //             (49, 102),
    //             (6, 57),
    //             (49, 83),
    //             (6, 83),
    //             (49, 57),
    //             (24, 95),
    //             (31, 45),
    //             (36, 111),
    //             (19, 29),
    //             (54, 126),
    //             (1, 14),
    //             (4, 11),
    //             (51, 129),
    //             (5, 36),
    //             (50, 104),
    //             (25, 87),
    //             (30, 53),
    //             (34, 88),
    //             (21, 52),
    //             (38, 100),
    //             (17, 40),
    //             (40, 112),
    //             (15, 28),
    //             (48, 139),
    //             (7, 1),
    //             (0, 0),
    //             (55, 140),
    //             (0, 0),
    //             (55, 140),
    //             (0, 0),
    //             (55, 140),
    //             (0, 0),
    //             (55, 140),
    //             (28, 22),
    //             (27, 118),
    //             (55, 140),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (58, 146),
    //             (79, 278),
    //             (0, 51),
    //             (10, 9),
    //             (5, 3),
    //             (0, 15),
    //             (8, 5),
    //             (157, 438),
    //             (3, 69),
    //             (158, 450),
    //             (2, 57),
    //             (160, 453),
    //             (0, 54),
    //             (160, 470),
    //             (0, 37),
    //             (160, 489),
    //             (0, 18),
    //             (160, 504),
    //             (0, 3),
    //             (126, 404),
    //             (34, 103),
    //             (126, 446),
    //             (34, 61),
    //             (126, 468),
    //             (34, 39),
    //             (126, 492),
    //             (34, 15),
    //             (126, 505),
    //             (34, 2),
    //             (129, 507),
    //             (31, 0),
    //             (143, 474),
    //             (17, 33),
    //             (145, 474),
    //             (15, 33),
    //             (145, 489),
    //             (15, 18),
    //             (145, 498),
    //             (15, 9),
    //             (149, 498),
    //             (11, 9),
    //             (149, 507),
    //             (11, 0),
    //             (159, 507),
    //             (1, 0),
    //             (67, 306),
    //             (93, 201),
    //             (10, 65),
    //             (150, 442),
    //             (26, 122),
    //             (134, 385),
    //             (26, 182),
    //             (134, 325),
    //             (27, 255),
    //             (133, 252),
    //             (67, 314),
    //             (93, 193),
    //             (103, 373),
    //             (57, 134),
    //             (147, 424),
    //             (13, 83),
    //             (28, 63),
    //             (132, 444),
    //             (54, 151),
    //             (106, 356),
    //             (97, 306),
    //             (63, 201),
    //             (115, 354),
    //             (45, 153),
    //             (132, 387),
    //             (28, 120),
    //             (135, 408),
    //             (25, 99),
    //             (146, 503),
    //             (14, 4),
    //             (68, 310),
    //             (92, 197),
    //             (96, 344),
    //             (64, 163),
    //             (103, 360),
    //             (57, 147),
    //             (132, 485),
    //             (28, 22),
    //             (160, 507),
    //             (0, 0),
    //             (154, 471),
    //             (2, 9),
    //             (4, 27),
    //         ],
    //         [
    //             (14, 38),
    //             (10, 73),
    //             (0, 0),
    //             (1, 0),
    //             (0, 2),
    //             (0, 4),
    //             (2, 1),
    //             (27, 113),
    //             (0, 5),
    //             (27, 116),
    //             (0, 2),
    //             (27, 116),
    //             (0, 2),
    //             (27, 116),
    //             (0, 2),
    //             (27, 116),
    //             (0, 2),
    //             (27, 116),
    //             (0, 2),
    //             (23, 98),
    //             (4, 20),
    //             (23, 112),
    //             (4, 6),
    //             (23, 116),
    //             (4, 2),
    //             (23, 118),
    //             (4, 0),
    //             (23, 118),
    //             (4, 0),
    //             (23, 118),
    //             (4, 0),
    //             (26, 113),
    //             (1, 5),
    //             (26, 113),
    //             (1, 5),
    //             (26, 117),
    //             (1, 1),
    //             (26, 117),
    //             (1, 1),
    //             (26, 117),
    //             (1, 1),
    //             (26, 118),
    //             (1, 0),
    //             (27, 118),
    //             (0, 0),
    //             (0, 0),
    //             (27, 118),
    //             (2, 14),
    //             (25, 104),
    //             (5, 36),
    //             (22, 82),
    //             (5, 55),
    //             (22, 63),
    //             (5, 74),
    //             (22, 44),
    //             (16, 86),
    //             (11, 32),
    //             (23, 92),
    //             (4, 26),
    //             (26, 105),
    //             (1, 13),
    //             (1, 9),
    //             (26, 109),
    //             (1, 26),
    //             (26, 92),
    //             (14, 69),
    //             (13, 49),
    //             (14, 70),
    //             (13, 48),
    //             (18, 81),
    //             (9, 37),
    //             (20, 92),
    //             (7, 26),
    //             (27, 117),
    //             (0, 1),
    //             (0, 0),
    //             (27, 118),
    //             (0, 0),
    //             (27, 118),
    //             (0, 0),
    //             (27, 118),
    //             (0, 0),
    //             (27, 118),
    //             (0, 0),
    //             (27, 118),
    //             (27, 118),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (72, 179),
    //             (83, 340),
    //             (0, 35),
    //             (11, 9),
    //             (5, 5),
    //             (0, 15),
    //             (10, 6),
    //             (178, 531),
    //             (3, 58),
    //             (179, 546),
    //             (2, 43),
    //             (181, 549),
    //             (0, 40),
    //             (181, 562),
    //             (0, 27),
    //             (181, 574),
    //             (0, 15),
    //             (181, 584),
    //             (0, 5),
    //             (149, 476),
    //             (32, 113),
    //             (149, 532),
    //             (32, 57),
    //             (149, 558),
    //             (32, 31),
    //             (149, 579),
    //             (32, 10),
    //             (149, 587),
    //             (32, 2),
    //             (152, 589),
    //             (29, 0),
    //             (163, 555),
    //             (18, 34),
    //             (165, 555),
    //             (16, 34),
    //             (165, 570),
    //             (16, 19),
    //             (165, 579),
    //             (16, 10),
    //             (169, 579),
    //             (12, 10),
    //             (169, 589),
    //             (12, 0),
    //             (180, 589),
    //             (1, 0),
    //             (61, 270),
    //             (120, 319),
    //             (12, 79),
    //             (169, 510),
    //             (31, 157),
    //             (150, 432),
    //             (31, 236),
    //             (150, 353),
    //             (32, 328),
    //             (149, 261),
    //             (83, 396),
    //             (98, 193),
    //             (126, 460),
    //             (55, 129),
    //             (173, 522),
    //             (8, 67),
    //             (29, 72),
    //             (152, 517),
    //             (55, 177),
    //             (126, 412),
    //             (108, 364),
    //             (73, 225),
    //             (123, 395),
    //             (58, 194),
    //             (144, 438),
    //             (37, 151),
    //             (149, 468),
    //             (32, 121),
    //             (167, 584),
    //             (14, 5),
    //             (62, 274),
    //             (119, 315),
    //             (90, 308),
    //             (91, 281),
    //             (97, 324),
    //             (84, 265),
    //             (126, 449),
    //             (55, 140),
    //             (154, 471),
    //             (27, 118),
    //             (181, 589),
    //             (0, 0),
    //             (0, 0),
    //         ],
    //         [
    //             (0, 0),
    //             (2, 1),
    //             (0, 6),
    //             (0, 0),
    //             (0, 0),
    //             (0, 2),
    //             (0, 0),
    //             (2, 3),
    //             (0, 6),
    //             (2, 3),
    //             (0, 6),
    //             (2, 3),
    //             (0, 6),
    //             (2, 4),
    //             (0, 5),
    //             (2, 6),
    //             (0, 3),
    //             (2, 9),
    //             (0, 0),
    //             (0, 8),
    //             (2, 1),
    //             (0, 8),
    //             (2, 1),
    //             (0, 8),
    //             (2, 1),
    //             (0, 9),
    //             (2, 0),
    //             (0, 9),
    //             (2, 0),
    //             (0, 9),
    //             (2, 0),
    //             (2, 7),
    //             (0, 2),
    //             (2, 7),
    //             (0, 2),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 1),
    //             (2, 8),
    //             (0, 2),
    //             (2, 7),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (2, 1),
    //             (0, 8),
    //             (2, 7),
    //             (0, 2),
    //             (2, 7),
    //             (0, 2),
    //             (2, 8),
    //             (0, 1),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //             (0, 0),
    //             (2, 9),
    //             (0, 0),
    //         ],
    //         [
    //             (0, 5),
    //             (4, 10),
    //             (0, 10),
    //             (0, 0),
    //             (0, 0),
    //             (0, 2),
    //             (0, 0),
    //             (4, 17),
    //             (0, 10),
    //             (4, 17),
    //             (0, 10),
    //             (4, 17),
    //             (0, 10),
    //             (4, 20),
    //             (0, 7),
    //             (4, 25),
    //             (0, 2),
    //             (4, 27),
    //             (0, 0),
    //             (0, 18),
    //             (4, 9),
    //             (0, 18),
    //             (4, 9),
    //             (0, 18),
    //             (4, 9),
    //             (0, 22),
    //             (4, 5),
    //             (0, 27),
    //             (4, 0),
    //             (0, 27),
    //             (4, 0),
    //             (4, 25),
    //             (0, 2),
    //             (4, 25),
    //             (0, 2),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (0, 0),
    //             (4, 27),
    //             (0, 1),
    //             (4, 26),
    //             (0, 1),
    //             (4, 26),
    //             (0, 1),
    //             (4, 26),
    //             (0, 4),
    //             (4, 23),
    //             (0, 4),
    //             (4, 23),
    //             (0, 5),
    //             (4, 22),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (1, 10),
    //             (3, 17),
    //             (4, 22),
    //             (0, 5),
    //             (4, 23),
    //             (0, 4),
    //             (4, 24),
    //             (0, 3),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (4, 27),
    //             (0, 0),
    //             (0, 0),
    //             (0, 0),
    //             (4, 27),
    //         ],
    //     ];
    //     let matrix = MurTree::build_depth_two_matrix(&mut structure, &candidates);
    //     assert_eq!(matrix.iter().eq(expected_matrix.iter()), true);
    //
    //     let mut structure = BitsetStructure::new(&bitset_data);
    //     let matrix = MurTree::build_depth_two_matrix(&mut structure, &candidates);
    //     assert_eq!(matrix.iter().eq(expected_matrix.iter()), true);
    //
    //     let data = HorizontalBinaryStructure::format_input_data(&dataset);
    //     let mut structure = HorizontalBinaryStructure::new(&data);
    //     let matrix = MurTree::build_depth_two_matrix(&mut structure, &candidates);
    //     assert_eq!(matrix.iter().eq(expected_matrix.iter()), true);
    // }
}
